@@ -33,19 +33,19 @@ use SrSv::Process::InParent qw(
 	whois_callback kill_callback
 );
 
-use SrSv::Conf 'main';
+use SrSv::Conf2Consts qw(main);
 
 use SrSv::Debug;
-use SrSv::Unreal::Tokens;
+use SrSv::Unreal::Tokens qw( :tokens );
 use SrSv::Unreal::Base64 qw(itob64);
-use SrSv::IRCd::State qw(synced $ircd_ready);
+use SrSv::IRCd::State qw(synced $ircd_ready %IRCd_capabilities);
 use SrSv::IRCd::IO qw(ircsend ircsendimm);
 use SrSv::IRCd::Event qw(addhandler);
-use SrSv::Unreal::Validate qw(valid_nick);
+use SrSv::IRCd::Validate qw(valid_nick);
 use SrSv::RunLevel 'main_shutdown';
 
 # FIXME
-BEGIN { *SJB64 = \&net::SJB64 }
+BEGIN { *SJB64 = \&ircd::SJB64 }
 
 our %agents;
 our @defer_join;
@@ -80,13 +80,13 @@ sub agent_connect($$$$$) {
 
 	$agents{lc $nick}{PARMS} = [ @_ ];
 
-	$host = $main_conf{local} unless $host;
-	ircsend($tkn{NICK}[$tkn]." $nick 1 $time $ident $host ".
-		(SJB64 ? itob64($main_conf{numeric}) : $main_conf{local}).
+	$host = main_conf_local unless $host;
+	ircsend("@{[TOK_NICK]} $nick 1 $time $ident $host ".
+		(SJB64 ? itob64(main_conf_numeric) : main_conf_local).
 		" 1 $modes * :$gecos");
 
 	foreach my $chan (@chans) {
-		ircsend(":$nick ".$tkn{JOIN}[$tkn]." $chan");
+		ircsend(":$nick @{[TOK_JOIN]} $chan");
 		# If we tracked chanmodes for agents, that would go here as well.
 	}
 }
@@ -97,7 +97,7 @@ sub agent_quit($$) {
 	delete($agents{lc $nick}{CHANS});
 	delete($agents{lc $nick});
 
-	ircsendimm(":$nick ".$tkn{QUIT}[$tkn]." :$msg");
+	ircsendimm(":$nick @{[TOK_QUIT]} :$msg");
 }
 
 sub agent_quit_all($) {
@@ -134,7 +134,7 @@ sub agent_join($$) {
 
 	if($agents{lc $agent}) {
 		$agents{lc $agent}{CHANS}{lc $chan} = 1;
-		ircsend(":$agent ".$tkn{JOIN}[$tkn]." $chan");
+		ircsend(":$agent @{[TOK_JOIN]} $chan");
 	} else {
 		if($ircd_ready) {
 			print "Tried to make nonexistent agent ($agent) join channel ($chan)" if DEBUG;
@@ -149,13 +149,13 @@ sub agent_part($$$) {
 	my ($agent, $chan, $reason) = @_;
 
 	delete($agents{lc $agent}{CHANS}{lc $chan});
-	ircsend(":$agent $tkn{PART}[$tkn] $chan :$reason");
+	ircsend(":$agent @{[TOK_PART]} $chan :$reason");
 }
 
 sub set_agent_umode($$) {
 	my ($src, $modes) = @_;
 
-	ircsend(":$src $tkn{UMODE2}[$tkn] $modes");
+	ircsend(":$src @{[TOK_UMODE2]} $modes");
 }
 
 sub agent_sync() {
@@ -188,30 +188,30 @@ sub whois_callback {
 	foreach my $nick (@nicks) {
 		if (is_agent($nick)) {
 			my ($nick, $ident, $host, $modes, $gecos) = @{$agents{lc $nick}{PARMS}};
-			$host = $main_conf{local} unless $host;
-			push @reply, ':'.$main_conf{local}." 311 $src $nick $ident $host * :$gecos";
-			push @reply, ':'.$main_conf{local}." 312 $src $nick ".$main_conf{local}.' :'.$main_conf{info};
+			$host = main_conf_local unless $host;
+			push @reply, ':'.main_conf_local." 311 $src $nick $ident $host * :$gecos";
+			push @reply, ':'.main_conf_local." 312 $src $nick ".main_conf_local.' :'.main_conf_info;
 			foreach my $mode (split(//, $modes)) {
 				if ($mode eq 'z') {
-					push @reply, ':'.$main_conf{local}." 671 $src $nick :is using a Secure Connection";
+					push @reply, ':'.main_conf_local." 671 $src $nick :is using a Secure Connection";
 				}
 				elsif($mode eq 'S') {
 					#313 tab ChanServ :is a Network Service
-					push @reply, ':'.$main_conf{local}." 313 $src $nick :is a Network Service";
+					push @reply, ':'.main_conf_local." 313 $src $nick :is a Network Service";
 				}
 				elsif($mode eq 'B') {
 					#335 blah2 TriviaBot :is a Bot on SurrealChat.net
-					push @reply, ':'.$main_conf{local}.
-						" 335 $src $nick :is a \002Bot\002 on ".$main_conf{network};
+					push @reply, ':'.main_conf_local.
+						" 335 $src $nick :is a \002Bot\002 on ".$IRCd_capabilities{NETWORK};
 				}
 			}
 		}
 		else {
-			push @reply, ':'.$main_conf{local}." 401 $src $nick :No such service";
+			push @reply, ':'.main_conf_local." 401 $src $nick :No such service";
 		}
 
 	}
-	push @reply, ':'.$main_conf{local}." 318 $src $nicklist :End of /WHOIS list.";
+	push @reply, ':'.main_conf_local." 318 $src $nicklist :End of /WHOIS list.";
 	ircsend(@reply);
 }
 
@@ -220,7 +220,7 @@ sub kill_callback($$$$) {
 	if (defined($agents{lc $dst})) {
 		if (defined ($agents{lc $dst}{KILLED}) and ($agents{lc $dst}{KILLED} == time())) {
 			if ($agents{lc $dst}{KILLCOUNT} > 3) {
-				net::debug("Caught in a kill loop for $dst, dying now.");
+				ircd::debug("Caught in a kill loop for $dst, dying now.");
 				main_shutdown;
 			} else {
 				$agents{lc $dst}{KILLCOUNT}++;
@@ -232,11 +232,11 @@ sub kill_callback($$$$) {
 
 		if($src =~ /\./) {
 			# let's NOT loopback this event
-			ircsendimm(':'.$main_conf{'local'}.' '.$tkn{KILL}[$tkn]." $dst :Nick Collision");
+			ircsendimm(':'.main_conf_local.' '."@{[TOK_KILL]} $dst :Nick Collision");
 		} elsif (defined($agents{lc $src})) {
 			# Do Nothing.
 		} else {
-			net::irckill($main::rsnick, $src, "Do not kill services agents.");
+			ircd::irckill($main::rsnick, $src, "Do not kill services agents.");
 		}
 
 		&agent_connect(@{$agents{lc $dst}{PARMS}}) if synced();
