@@ -21,15 +21,17 @@ use strict;
 use SrSv::SimpleHash qw(read_hash);
 
 our %conffiles;
-our $prefix;
 
+=cut
+our $prefix;
 BEGIN {
-	if($main::prefix) {
-		$prefix = $main::prefix;
+	if(main::PREFIX()) {
+		$prefix = main::PREFIX();
 	} else {
 		$prefix = '.';
 	}
 }
+=cut
 
 sub install_conf($$) {
 	no strict 'refs';
@@ -43,7 +45,7 @@ sub import {
 
 	foreach my $file (@files) {
 		unless(defined $conffiles{$file}) {
-			$conffiles{$file} = { read_hash("$prefix/config/$file.conf") };
+			$conffiles{$file} = { read_hash(main::PREFIX()."/config/$file.conf") };
 		}
 
 		install_conf(caller(), $file);
