@@ -23,6 +23,8 @@ BEGIN { our @EXPORT_OK = qw(addhandler callfuncs) }
 
 use SrSv::Debug;
 
+use SrSv::Conf2Consts qw( main );
+
 use SrSv::IRCd::Queue qw(ircd_enqueue);
 use SrSv::IRCd::State qw($ircline $ircline_real synced initial_synced);
 
@@ -59,6 +61,7 @@ sub addhandler($$$$;$) {
 	});
 }
 
+our $last_highqueue = time();
 sub callfuncs {
 	my ($args, $sync, $wf, $message);
 
@@ -83,7 +86,15 @@ sub callfuncs {
 		ARGS => $args,
 		ON_FINISH => ($sync ? undef : 'SrSv::IRCd::Queue::finished'), # FIXME
 		SYNCED => [synced, initial_synced],
+		QUEUE_DEPTH => SrSv::IRCd::Queue::queue_size(),
 	};
+	if(initial_synced && ($message->{QUEUE_DEPTH} > main_conf_highqueue) && ($last_highqueue < time()-5)) {
+		ircd::privmsg_noloop(main_conf_local, main_conf_operchan, "HIGH TRAFFIC WARNING",
+			"Queue depth exceeded @{[main_conf_highqueue]}") if defined(main_conf_operchan);
+		ircd::privmsg_noloop(main_conf_local, main_conf_diag, "HIGH TRAFFIC WARNING",
+			"Queue depth exceeded @{[main_conf_highqueue]}");
+		$last_highqueue = time();
+	}
 
 	if($sync) {
 		message($message);
@@ -98,15 +109,16 @@ sub _realcall($$) {
 	my ($message, $callback) = @_;
 
 	print "Calling ", $callback->{REALCALL}, " ", join(',', @{$message->{ARGS}}), "\n" if DEBUG();
-	$ircline = $message->{IRCLINE};
+	local $ircline = $message->{IRCLINE};
 
 	local $SrSv::IRCd::State::synced = $message->{SYNCED}[0]; # XXX This is questionable.
 	local $SrSv::IRCd::State::initial_synced = $message->{SYNCED}[1];
+	local $SrSv::IRCd::State::queue_depth = $message->{QUEUE_DEPTH};
 
 	print "IRCLINE is $ircline  synced is $SrSv::IRCd::State::synced  initial_synced is $SrSv::IRCd::State::initial_synced\n" if DEBUG();
 
 	&{$callback->{REALCALL}}(@{$message->{ARGS}});
-	net::flushmodes() unless $message->{SYNC}; # FIXME
+	ircd::flushmodes() unless $message->{SYNC}; # FIXME
 	print "Finished with $ircline\n" if DEBUG();
 }
 
