@@ -33,9 +33,10 @@ use SrSv::Process::Worker qw(ima_worker);
 use SrSv::Debug;
 use SrSv::IRCd::State qw($ircline $ircline_real $ircd_ready);
 use SrSv::IRCd::Event qw(callfuncs);
-use SrSv::Unreal::Tokens;
-use SrSv::Unreal::Parse qw(parse_line);
+use SrSv::Unreal::Tokens qw( :tokens );
+use SrSv::IRCd::Parse qw(parse_line);
 use SrSv::RunLevel qw(emerg_shutdown);
+use SrSv::Log qw( write_log );
 
 our $irc_sock;
 our @queue;
@@ -68,7 +69,7 @@ sub irc_error($) {
 
 		foreach my $line (@lines) {
 			$ircline_real++ unless $line =~ /^(?:8|PING)/;
-			log::write_log('netdump', '', $line) if main::NETDUMP();
+			write_log('netdump', '', $line) if main::NETDUMP();
 			print ">> $ircline_real $line\n" if DEBUG_ANY;
 			foreach my $ev (parse_line($line)) {
 				next unless $ev;
@@ -90,8 +91,9 @@ sub irc_error($) {
 			PeerAddr => $remote,
 			PeerPort => $port,
 			Proto => 'tcp',
-			Blocking => 0,
-		) or die("Could not connect to IRC server: $!");
+			Blocking => 1,
+		) or die("Could not connect to IRC server ($remote:$port): $!");
+		$irc_sock->blocking(0);
 		print " done\n" if DEBUG;
 
 		$irc_sock->autoflush(1);
@@ -123,7 +125,7 @@ sub ircsendimm {
 		my $r;
 		my $bytes = 0;
 		my $len = length($line) + 2;
-		log::write_log('netdump', '', split(NL, $line))
+		write_log('netdump', '', split(NL, $line))
 			if main::NETDUMP();
 		while(1) {
 			$r = $irc_sock->syswrite($line . NL, undef, $bytes);
@@ -171,7 +173,7 @@ sub ircsend {
 		ircsendimm(@_);
 	} else {
 		foreach my $x (@_) {
-			if($x =~ /^$tkn{NICK}[$tkn]/) {
+			if($x =~ /^@{[TOK_NICK]}/) {
 				unshift @queue, $x;
 			} else {
 				push @queue, $x;
