@@ -23,7 +23,12 @@ use Exporter 'import';
 our @EXPORT_OK = qw($ircline $ircline_real $remoteserv $ircd_ready synced initial_synced create_server get_server_children set_server_state set_server_juped get_server_state get_online_servers %IRCd_capabilities);
 
 # FIXME - synced() is called very often and should be cached locally
-use SrSv::Process::InParent qw(calc_synced create_server get_server_children set_server_state set_server_juped get_server_state get_online_servers);
+use SrSv::Process::InParent qw(
+	calc_synced
+	__initial_synced_inparent __synced_inparent
+	create_server get_server_children
+	set_server_state set_server_juped
+	get_server_state get_online_servers);
 
 use SrSv::Conf 'main';
 
@@ -40,17 +45,26 @@ our %servers;
 our %juped_servers;
 our $synced;
 our $initial_synced;
+our $queue_depth;
 
-sub synced {
+sub __initial_synced_inparent {
+	return $initial_synced;
+}
+sub __synced_inparent {
 	return $synced;
 }
 
+sub synced {
+# $ircline is zero if running in a timer context (among other possibilities)
+	return ($ircline ? $synced : __synced_inparent());
+}
+
 sub initial_synced {
-	return $initial_synced;
+	return ($ircline ? $initial_synced : __synced_inparent());
 }
 
 sub calc_synced {
-	#return ($sync and $sync < $net::ircline);
+	#return ($sync and $sync < $ircd::ircline);
 
 	SYNCED: {
 		foreach my $s (keys(%servers)) {
@@ -84,6 +98,7 @@ sub create_server($$) {
 		PARENT => $parent,
 		CHILDREN => [],
 		SYNCED => 0,
+		NONCONFORMANT => isNonconformant($parent, $child),
 	};
 
 	push @{$servers{$parent}{CHILDREN}}, $child if $parent;
@@ -127,11 +142,31 @@ sub set_server_juped($) {
 	$juped_servers{$server} = 1;
 }
 
+sub isNonconformant(@) {
+	my (@serverList) = @_;
+	foreach my $server (@serverList) {
+		if(defined($servers{$server}) && $servers{$server}->{NONCONFORMANT}) {
+			return 1;
+		}
+		if(defined $main_conf{'unsyncserver'}) {
+			my @list;
+			if(ref($main_conf{'unsyncserver'}) eq 'ARRAY') {
+				@list = @{$main_conf{'unsyncserver'}};
+			} else {
+				@list = ($main_conf{'unsyncserver'});
+			}
+			if(grep (m/^$server$/i, @list) ) {
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
 sub get_server_state {
 	my ($server) = @_;
 
-	my $badserver = $main_conf{'unsyncserver'};
-	return 1 if($badserver and lc $server eq lc $badserver); # I HATE NEOSTATS
+	return 1 if isNonconformant($server);
 
 	return $servers{$server}{SYNCED};
 }
