@@ -204,9 +204,9 @@ sub callback_matches($$) {
 
 	foreach my $cond (keys(%{$callback->{TRIGGER_COND}})) {
 		if(ref($callback->{TRIGGER_COND}{$cond}) eq 'Regexp') {
-			return 0 unless $message->{$cond} =~ $callback->{TRIGGER_COND}{$cond};
+			return 0 if defined($message->{$cond}) && !($message->{$cond} =~ $callback->{TRIGGER_COND}{$cond});
 		} else {
-			return 0 unless lc $message->{$cond} eq lc $callback->{TRIGGER_COND}{$cond};
+			return 0 if defined($message->{$cond}) && !(lc $message->{$cond} eq lc $callback->{TRIGGER_COND}{$cond});
 		}
 	}
 
