@@ -18,17 +18,26 @@ package SrSv::MySQL;
 
 use strict;
 
-use Exporter 'import';
-BEGIN { our @EXPORT_OK = qw($dbh) }
+use DBI qw( :sql_types );
 
-use DBI;
+use Exporter 'import';
+BEGIN {
+	our @EXPORT_OK = (qw( $dbh connectDB disconnectDB ), @{$DBI::EXPORT_TAGS{'sql_types'}} );
+	our %EXPORT_TAGS = ( sql_types => $DBI::EXPORT_TAGS{'sql_types'} );
+}
 
 use SrSv::Process::Init;
+
+use SrSv::Conf::sql;
+
 use SrSv::Conf 'sql';
 
 our $dbh;
 
 proc_init {
+	connectDB();
+};
+sub connectDB() {
 	$dbh = DBI->connect(
 		"DBI:mysql:".$sql_conf{'mysql-db'}.($sql_conf{server_prepare} ? ":mysql_server_prepare=1" : ''),
 		$sql_conf{'mysql-user'},
@@ -41,6 +50,11 @@ proc_init {
 	);
 	# Prevent timeout
 	$dbh->do("SET wait_timeout=(86400*365)");
-};
+}
+
+sub disconnectDB() {
+	$dbh->disconnect();
+	$dbh = undef;
+}
 
 1;
