@@ -32,12 +32,13 @@ sub safe_call($$) {
 		no strict 'refs';
 
 		local $SIG{__WARN__} = sub {
-			net::debug(" -- Warning: ".$_[0]);
+			ircd::debug(" -- Warning: ".$_[0],
+				($_[0] =~ /MySQL\/Stub/ ? split(/\n/, Carp::longmess($@)) : undef ) );
 		};
 
 		local $SIG{__DIE__} = sub {
 			($_[0] =~ /^user/) or
-				net::debug(" --", "-- DIED: ".$_[0], split(/\n/, Carp::longmess($@)), " --");
+				ircd::debug(" --", "-- DIED: ".$_[0], split(/\n/, Carp::longmess($@)), " --");
 		};
 			
 
