@@ -21,13 +21,24 @@ use strict;
 use Carp 'croak';
 
 use Exporter 'import';
-BEGIN { our @EXPORT_OK = qw(spawn ima_worker $ima_worker multi get_socket call_in_parent call_all_child do_callback_in_child shutdown_worker shutdown_all_workers kill_all_workers) }
+BEGIN {
+	our @EXPORT_OK = qw(spawn write_pidfiles
+		ima_worker $ima_worker
+		multi get_socket
+		call_in_parent call_all_child do_callback_in_child
+		shutdown_worker shutdown_all_workers kill_all_workers)
+	}
 
 use Event;
+use English qw( -no_match_vars );
 use IO::Socket;
+use IO::File;
 use Storable qw(fd_retrieve store_fd);
 
 use SrSv::Debug;
+
+sub PREFIX() { return main::PREFIX }
+
 BEGIN {
 	if(DEBUG) {
 		require Data::Dumper; import Data::Dumper ();
@@ -77,6 +88,15 @@ sub spawn() {
 	}
 }
 
+sub write_pidfiles() {
+	my $fh = IO::File->new("@{[PREFIX]}/data/worker.pids", 'w', '0600');
+	for(my $i = scalar(@workers); $i; $i--) {
+		my $pid = $workers[$i-1]->{PID};
+		print $fh $pid,"\n";
+	}
+	print $fh $PID,"\n";
+}
+
 sub ima_worker {
 	return $ima_worker;
 }
