@@ -62,7 +62,7 @@ sub emerg_shutdown() {
 }
 
 sub _main_shutdown() {
-	net::agent_quit_all("Shutting down.");
+	ircd::agent_quit_all("Shutting down.");
 
 	emerg_shutdown;
 }
