@@ -22,29 +22,29 @@ use Exporter 'import';
 BEGIN { our @EXPORT_OK = qw(read_hash readHash write_hash writeHash) }
 
 sub writeHash {
-        my $hash = $_[0];
-        my $file = $_[1];
+	my $hash = $_[0];
+	my $file = $_[1];
 
 	my $fh;
-        open $fh, ">$file";
+	open $fh, '>', $file;
 
-        my @keys = keys(%$hash); my @values = values(%$hash);
+	my @keys = keys(%$hash); my @values = values(%$hash);
 
-        for(my $i=0; $i<@keys; $i++) {
-                if($values[$i][0]) {
-                        chomp $keys[$i];
-                        print $fh $keys[$i], " =[ ";
-                        foreach my $atom (@{$values[$i]}) {
-                                print $fh $atom, ", ";
-                        }
-                        print $fh "\n";
-                } else {
-                        chomp $keys[$i]; chomp $values[$i];
-                        print $fh $keys[$i], " = ", $values[$i], "\n";
-                }
-        }
+	for(my $i=0; $i<@keys; $i++) {
+		if(ref($values[$i]) eq 'ARRAY') {
+			chomp $keys[$i];
+			print $fh $keys[$i], " =[ ";
+			foreach my $atom (@{$values[$i]}) {
+				print $fh $atom, ", ";
+			}
+			print $fh "\n";
+		} else {
+			chomp $keys[$i]; chomp $values[$i];
+			print $fh $keys[$i], " = ", $values[$i], "\n";
+		}
+	}
 
-        close $fh;
+	close $fh;
 }
 
 sub readHash {
@@ -56,24 +56,31 @@ sub readHash {
 		or die "ERROR: Unable to open config file $file: $!\n";
 
 	while(my $line = <$fh>) {
-                if($line =~ / =\[ /) {
-                        my ($key, $value) = split(/ =\[ /, $line);
-                        chomp $key; chomp $value;
-                        $hash{$key} = [ split(/, /, $value) ];
-                }
-		elsif($line =~ / = /) {
-                        my ($key, $value) = split(/ = /, $line);
-                        chomp $key; chomp $value;
-                        $hash{$key} = $value;
-                }
-		elsif($line =~ /^#|^\s*$/) { }
+		if($line =~ /^#|^\s*$/) { }
+		elsif($line =~ /^(\S+) ?= ?\[ ?(.*) ?]$/) {
+			my ($key, $value) = ($1, $2);
+			chomp $key; chomp $value;
+			$key =~ s/(^\s+|\s+$)//g;
+			$value =~ s/(^\s+|\s+$)//g;
+			$hash{$key} = [ split(/, /, $value) ];
+		}
+		elsif($line =~ /^\S+ ?= ?/) {
+			my ($key, $value) = split(/ ?= ?/, $line, 2);
+			chomp $key; chomp $value;
+			if($value eq 'undef') {
+				$value = undef;
+			}
+			$key =~ s/(^\s+|\s+$)//g;
+			$value =~ s/(^\s+|\s+$)//g;
+			$hash{$key} = $value;
+		}
 		else {
 			die "Malformed config file: $file\n";
 		}
-        }
-        close $fh;
+	}
+	close $fh;
 
-        return (%hash);
+	return (%hash);
 }
 
 BEGIN { # The same functions, now with less camelCase
