@@ -37,7 +37,7 @@ add_callback({
 
 if(DEBUG()) {
 	add_timer('hello', 2, __PACKAGE__, 'SrSv::Timer::test');
-	sub test { net::privmsg('ServServ', '#surrealchat', $_[0]) };
+	sub test { ircd::privmsg('ServServ', '#surrealchat', $_[0]) };
 }
 
 sub add_timer($$$$) {
