@@ -18,13 +18,19 @@ package SrSv::Unreal::Tokens;
 
 use strict;
 
+use SrSv::Conf::main;
+use SrSv::Conf2Consts qw( main );
+
 use Exporter 'import';
-BEGIN { our @EXPORT = qw($tkn %tkn) }
 
-our $tkn = 0;
+#=cut
+our $tkn = main_conf_tokens;
 
+our %tkn;
+#=cut
+BEGIN {
 # TODO: Turn these into constants.
-our %tkn = (
+my %unrealTokens = (
 	PRIVMSG 	=> ['PRIVMSG',	'!'],
 	WHOIS 		=> ['WHOIS',	'#'],
 	WHOWAS		=> ['WHOWAS',	'$'],
@@ -88,7 +94,33 @@ our %tkn = (
 	EOS		=> ['EOS',	'ES'],
 	UMODE2		=> ['UMODE2',	"\|"],
 
-	REHASH		=> ['REHASH',	'O']
+	REHASH		=> ['REHASH',	'O'],
+
+	SVSNOLAG	=> ['SVSNOLAG', 'sl'],
+	SVS2NOLAG	=> ['SVS2NOLAG', 'SL'],
 );
 
+	%tkn = %unrealTokens;
+	my %msgs; map { $msgs{"MSG_$_"} = $unrealTokens{$_}->[0] } keys(%unrealTokens);
+	my %toks;
+	if(main_conf_tokens) {
+		map { $toks{"TOK_$_"} = $unrealTokens{$_}->[1] } keys(%unrealTokens);
+	} else {
+		map { $toks{"TOK_$_"} = $unrealTokens{$_}->[0] } keys(%unrealTokens);
+	}
+	require constant;
+	import constant \%toks;
+	import constant \%msgs;
+
+	our @EXPORT_OK = (
+		keys(%toks),
+		keys(%msgs),
+		qw( %tkn $tkn )
+	);
+	our %EXPORT_TAGS = (
+		tokens => [keys(%toks)],
+		messages => [keys(%msgs)],
+	);
+}
+
 1;
