@@ -17,11 +17,19 @@
 #	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 use DBI;
-use Cwd 'abs_path';
-use File::Basename;
-use libs::config;
 
-my $prefix = dirname(abs_path($0));
+BEGIN {
+	use Cwd qw( abs_path getcwd );
+	use File::Basename;
+	my %constants = (
+		CWD => getcwd(),
+		PREFIX => dirname(abs_path($0)),
+	);
+	require constant; import constant(\%constants);
+	chdir $constants{PREFIX};
+}
+use lib PREFIX;
+		
 use SrSv::Conf 'sql';
 
 $dbh = DBI->connect('DBI:mysql:'.$sql_conf{'mysql-db'}, $sql_conf{'mysql-user'}, $sql_conf{'mysql-pass'}, 
