@@ -16,37 +16,87 @@
 #	along with SurrealServices; if not, write to the Free Software
 #	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 use strict;
-use DBI;
 
 use Getopt::Long;
 
-use modules::serviceslibs::svsdb;
-use Cwd 'abs_path';
-use File::Basename;
- 
-our $delete_db;
-GetOptions (
-	"delete" => \$delete_db,
-);
+BEGIN {
+	use Cwd qw( abs_path getcwd );
+	use File::Basename;
+	my %constants = (
+		CWD => getcwd(),
+		PREFIX => dirname(abs_path($0)),
+	);
+	require constant; import constant(\%constants);
+}
 
-my $prefix = dirname(abs_path($0));
+chdir PREFIX;
+use lib PREFIX;
 
-use SrSv::Conf 'sql';
+our ($delete_db, $skip_backup, $auto_backup, $restore, $help);
+BEGIN {
+	GetOptions (
+		"delete" => \$delete_db,
+		"skip-backup" => \$skip_backup,
+		"backup" => \$auto_backup,
+		"restore" => \$restore,
+		"help" => \$help,
+	);
 
-print "Connecting...\n";
+	if($help) {
+		print qq"
+Options:
+	--delete	Delete entire database
+	--skip-backup	Don't nag about making a backup
+	--backup	Make backup without upgrading
+	--restore FILE	Restore the database from a backup
+	--help		Show this message
+";
 
-my $dbh;
-eval { $dbh = DBI->connect("DBI:mysql:".$sql_conf{'mysql-db'}, $sql_conf{'mysql-user'}, $sql_conf{'mysql-pass'}, {  AutoCommit => 1, RaiseError => 1 }) };
+		exit 1;
+	}
+}
 
-if($@) {
-	print "FATAL: Can't connect to database:\n$@\n";
-	print "You must edit config/services.conf and create a corresponding\nMySQL user and database!\n\n";
-	exit;
+use SrSv::Conf qw(main sql);
+use SrSv::DB::Schema;
+
+BEGIN {
+	if($restore) {
+		my $f = shift @ARGV;
+		$f or die "You must specify a backup file to restore.\n";
+		print "Restoring from backup...\n";
+		system("mysql $sql_conf{'mysql-db'} -u $sql_conf{'mysql-user'} --password=$sql_conf{'mysql-pass'} <$f");
+		print "Finished.\n";
+		exit;
+	}
+}
+
+use SrSv::MySQL '$dbh';
+
+use SrSv::Upgrade::HashPass;
+
+my $backup_file;
+
+sub ask($) {
+	print shift;
+
+	while(my $c = getc) {
+		next unless $c =~ /\S/;
+		return (lc $c eq 'y');
+	}
+}
+
+unless($skip_backup) {
+	if($auto_backup or ask "Would you like to make a backup of your database: $sql_conf{'mysql-db'}? (Y/n) ") {
+		my @lt = localtime();
+		$backup_file = "./db-backup-" . sprintf( "%04d%02d%02d", ($lt[5]+1900) , ($lt[4]+1) , ($lt[3]) ) . "-$$.sql";
+		print "Creating backup in $backup_file\n";
+		system("./utils/db-dump.pl > $backup_file");
+		goto END if $auto_backup;
+	}
 }
 
 if($delete_db) {
-	print "Really delete all data in database: $sql_conf{'mysql-db'}? (y/N) ";
-	exit unless lc getc eq 'y';
+	exit unless ask "Really delete all data in database: $sql_conf{'mysql-db'}? (y/N) ";
 
 	print "Deleting old tables...\n";
 
@@ -57,16 +107,17 @@ if($delete_db) {
 	}
 }
 
-print "Creating tables...\n";
-
 $dbh->{RaiseError} = 0;
 $dbh->{PrintError} = 0;
 
-open SQL, "sql/services.sql";
-my $sql;
-while(my $x = <SQL>) { $sql .= $x unless $x =~ /^#/ or $x eq '\n'}
-foreach my $line (split(/;/s, $sql)) {
-	$dbh->do($line);
+my ($ver) = check_schema();
+#print "$ver\n";
+if($ver == 0) {
+	print "Creating tables...\n";
+	do_sql_file("sql/services.sql");
+	upgrade_schema(0);
+} elsif($ver) {
+	upgrade_schema($ver);
 }
 
 print "Updating chanperm...\n";
@@ -92,8 +143,8 @@ my @perms = (
 	['INVITE', 4, 0],
 	['InviteSelf', 1, 0],
 	['TOPIC', 5, 0],
-	['UnbanSelf', 4, 0],
-	['UNBAN', 5, 0],
+	['UnbanSelf', 2, 0],
+	['UNBAN', 4, 0],
 	['VOICE', 2, 0],
 	['HALFOP', 3, 0],
 	['OP', 4, 0],
@@ -109,6 +160,9 @@ my @perms = (
 	['WELCOME', 6, 0],
 	['DICE', 1, 0],
 	['UPDOWN', 1, 0],
+	['MemoAccChange', 8, 0],
+	['MODE', 6, 0],
+	['COPY', 7, 0],
 );
 
 my @noperms = ();
@@ -121,9 +175,9 @@ foreach my $p (@noperms) {
 	$del_perm->execute($p);
 }
 
-print "Updating nick IDs...\n";
-
-$svsdb::dbh = $dbh;
-svsdb::nrid_convert();
+hash_all_passwords();
 
 print "Database setup complete!\n";
+
+END:
+$backup_file and print "\nNOTE: To restore your backup, use this command:\n  ./db-setup.pl --restore $backup_file\n";
