@@ -50,13 +50,15 @@ sub parse_quoted($) {
 	return @out;
 }
 
+use constant { ORD_A => ord('A') };
+
 sub gen_uuid($$) {
 	my ($groups, $length) = @_;
 	my $emailreg_code = '';
 	for(my $i = 1; $i <= $groups; $i++) {
 		for (my $j = 1; $j <= $length; $j++) {
 			my $ch;
-			$emailreg_code .= (($ch = int(rand(36))) > 9 ? chr(ord('A') + $ch - 10) : $ch);
+			$emailreg_code .= (($ch = int(rand(36))) > 9 ? chr((ORD_A - 10) + $ch) : $ch);
 		}
 		$emailreg_code .= '-' unless $i >= $groups;
 	}
