@@ -62,7 +62,7 @@ sub sanitize($) {
 		if($m eq '-') { $sign = 0; next; }
 		
 		if($sign) {
-			if($m =~ $net::ocm) {
+			if($m =~ $ircd::ocm) {
 				$parms[$c] = '*' if $m eq 'k';
 				$c++;
 			}
@@ -82,7 +82,7 @@ sub get_key($) {
 		if($m eq '-') { $sign = 0; next; }
 		
 		if($sign) {
-			if($m =~ $net::ocm) {
+			if($m =~ $ircd::ocm) {
 				return $parms[$c] if ($m eq 'k');
 				$c++;
 			}
@@ -97,7 +97,7 @@ sub get_key($) {
 #  'bekfLlvhoaq' 'kfLlj'
 ######
 # This really needs to be made more generic
-# learn more about $net::ocm $net::scm $net::acm
+# learn more about $ircd::ocm $ircd::scm $ircd::acm
 ######
 sub calc($$$$) {
 	my ($src, $dst, $chan, $type) = @_;
@@ -116,15 +116,21 @@ sub calc($$$$) {
 	foreach my $x (@smodes) {
 		if($x eq '+') { $sign=2; next; }
 		if($x eq '-') { $sign=1; next; }
-		if($chan and $x =~ $net::scm) {
+		if($chan and $x =~ $ircd::scm) {
 			#shift @sargs if($sign == 2);
 			my $t = shift @sargs;
 			if($type == MERGE) {
-				$status{$x}{lc $t} = $sign;
+				my $key;
+				if($x =~ /^[beIk]$/) {
+					$key = $t;
+				} else {
+					$key = lc $t;
+				}
+				$status{$x}{$key} = $sign;
 			}
 			next;
 		}
-		if($chan and $x !~ $net::acm) {
+		if($chan and $x !~ $ircd::acm) {
 			next;
 		}
 	
@@ -132,14 +138,14 @@ sub calc($$$$) {
 			$tmodes[ord($x)] = $sign if $type == DIFF;
 			$omodes[ord($x)] = $sign if $type == ADD;
 			
-			if($chan and $sign == 2 and $x =~ $net::ocm) {
+			if($chan and $sign == 2 and $x =~ $ircd::ocm) {
 				$targs[ord($x)] = shift @sargs if $type == DIFF;
 				$oargs[ord($x)] = shift @sargs if $type == ADD;
 			}
 		}
 		
 		elsif($type == MERGE) {
-			if($chan and $sign == 2 and $x =~ $net::ocm) {
+			if($chan and $sign == 2 and $x =~ $ircd::ocm) {
 				if(
 					($x eq 'l' and $sargs[0] =~ /^\d+$/) or
 					($x eq 'L' and $sargs[0] =~ /^#/) or
@@ -158,19 +164,25 @@ sub calc($$$$) {
 	foreach my $x (@dmodes) {
 		if($x eq '+') { $sign=2; next; }
 		if($x eq '-') { $sign=1; next; }
-		if($chan and $x =~ $net::scm) {
+		if($chan and $x =~ $ircd::scm) {
 			#shift @dargs if($sign == 2);
 			my $t = shift @dargs;
 			if($type == MERGE) {
-				$status{$x}{lc $t} = $sign;
+				my $key;
+				if($x =~ /^[beIk]$/) {
+					$key = $t;
+				} else {
+					$key = lc $t;
+				}
+				$status{$x}{$key} = $sign;
 			}
 			next;
 		}
-		if($chan and $x !~ $net::acm) {
+		if($chan and $x !~ $ircd::acm) {
 			next;
 		}
 
-		if($chan and $sign == 2 and $x =~ $net::ocm) {
+		if($chan and $sign == 2 and $x =~ $ircd::ocm) {
 			$oargs[ord($x)] = shift @dargs;
 		}
 
@@ -194,7 +206,7 @@ sub calc($$$$) {
 	}
 
 	$sign = 0;
-	for(my $i; $i<@omodes; $i++) {
+	for(my $i = 0; $i < scalar @omodes; $i++) {
 		if($omodes[$i] == 2) {
 			if($sign != 2) { $sign = 2; $rmodes .= '+'; }
 			$rmodes .= chr($i);
@@ -216,7 +228,7 @@ sub calc($$$$) {
 	}
 	
 	if($type == DIFF or $type == MERGE) {
-		for(my $i; $i<@omodes; $i++) {
+		for(my $i = 0; $i < scalar @omodes; $i++) {
 			if($omodes[$i] == 1) {
 				if($sign != 1) { $sign = 1; $rmodes .= '-'; }
 				$rmodes .= chr($i);
@@ -244,23 +256,23 @@ sub calc($$$$) {
 }
 
 # Splits modes into a hash
-# Skips modes in $net::scm (opmodes and banmodes)
+# Skips modes in $ircd::scm (opmodes and banmodes)
 sub splitmodes($) {
 	my ($modes) = @_;
-	my %modelist;
-	my ($modes, @parms) = split(/ /, $modes);
+	my (%modelist, @parms);
+	($modes, @parms) = split(/ /, $modes);
 	my $sign = '+';
 	foreach my $mode (split(//, $modes)) {
 		if ($mode eq '+' or $mode eq '-') {
 			$sign = $mode;
 		}
-		elsif($mode =~ $net::scm) {
+		elsif($mode =~ $ircd::scm) {
 			shift @parms;
 		}
-		elsif($mode =~ $net::ocm) {
+		elsif($mode =~ $ircd::ocm) {
 			push @{$modelist{$mode}}, $sign, shift @parms;
 		}
-		elsif($mode =~ $net::acm) {
+		elsif($mode =~ $ircd::acm) {
 			push @{$modelist{$mode}}, $sign;
 		}
 	}
@@ -308,10 +320,19 @@ our %unsafeumodes = (
 sub allowed_umodes($) {
 	my ($modes) = @_;
 	my %modelist = splitumodes($modes);
+	my ($rejected, $rejectedSign);
 	foreach my $mode (keys(%modelist)) {
-		$modelist{$mode} = undef if defined ($unsafeumodes{$mode});
+		if(defined ($unsafeumodes{$mode})) {
+			if(defined($rejectedSign) && $rejectedSign eq $modelist{$mode}) {
+			} else {
+				$rejectedSign = $modelist{$mode};
+				$rejected .= $rejectedSign;
+			}
+			$modelist{$mode} = undef;
+			$rejected .= $mode;
+		}
 	}
-	return unsplit_umodes(%modelist);
+	return (unsplit_umodes(%modelist), $rejected);
 }
 
 # split + unsplit equals a modes::merge for umodes
@@ -335,48 +356,4 @@ sub merge_umodes($;$) {
 	return modes::unsplit_umodes(modes::splitumodes($umodes1 . ($umodes2 ? $umodes2 : '' ) ) );
 }
 
-sub validate_chmodej($) {
-	my ($joins, $seconds) = split(/:/, @_);
-	return 0 unless (defined $joins and ($joins <= 255 and $joins >=1));
-	return 0 unless (defined $seconds and ($seconds <= 999 and $seconds >=1));
-	return 1;
-}
-
-my %chmodef_types = (
-	c => [{'m' => 1, 'M' => 1}, 0, 60],
-	j => [{'R' => 1}, 0, 60],
-	k => -1,
-	m => [{'M' => 1}, 0, 60],
-	n => -1,
-	t => [{'b' => 1}, -1],
-);
-
-sub validate_chmodef($) {
-	my ($block, $seconds) = split(/:/, @_);
-	# [4j#i5,3k#K7,15m#M10,5n#N5,6t#b]:5
-	
-	return 0 unless (defined($seconds) and ($seconds <= 999 and $seconds > 0));
-
-	$block =~ s/(\[|\])//g;
-
-	foreach my $tuple (split(',', $block)) {
-		my ($limit, $action) = split('#', $tuple);
-		my ($type, $time);
-		{
-			$limit =~ /([a-z])([0-9]{1,3})/;
-			($type, $time) = ($1, $2);
-		}
-		return 0 unless defined($chmodef_types{$type});
-
-
-		my $restrictions = $chmodef_types{$limit};
-		if($restrictions == -1) {
-			return 0 if defined($action);
-		} else {
-			my ($alt, $time) = split(//, $action, 2);
-			return 0 if (defined($action) and $restrictions->[0]->{$a});
-		}
-	}
-}
-
 1;
