@@ -19,7 +19,7 @@ no strict 'refs';
 
 use Symbol qw(delete_package);
 
-use SrSv::Conf 'main';
+use SrSv::Conf2Consts qw(main);
 
 use constant {
 	ST_UNLOADED => 0,
@@ -31,15 +31,15 @@ our %modules;
 our %packages;
 our @modules;
 
-our @unload : shared;
-our @load : shared;
+our @unload;
+our @load;
 
 sub load(@) {
 	my @m = @_;
-	@m = @modules = split(/\s*,\s*/, $main_conf{load}) unless @m;
+	@m = @modules = split(/\s*,\s*/, main_conf_load) unless @m;
 
 	foreach my $module (@m) {
-		next unless $modules{$module}[0] == ST_UNLOADED;
+		next if ($modules{$module} and $modules{$module}[0] and !($modules{$module}[0] == ST_UNLOADED));
 		
 		my $m = "./modules/$module.pm";
 		print "Loading module $module..."; # if $main::status==main::ST_PRECONNECT();
