@@ -18,8 +18,17 @@ package connectserv;
 use strict;
 no strict 'refs';
 
-use SrSv::IRCd::State 'synced';
+use SrSv::IRCd::State qw( initial_synced synced );
 use SrSv::IRCd::Event 'addhandler';
+use SrSv::RunLevel qw( $runlevel :levels );
+
+use SrSv::Conf::Parameters connectserv => [
+	[ joinpart => 0 ],
+];
+
+use SrSv::Conf2Consts qw( main connectserv );
+
+use SrSv::Log;
 
 use SrSv::Process::InParent qw(
 	ev_nickconn ev_nickchange ev_quit ev_kill ev_umode ev_connect message
@@ -32,16 +41,16 @@ use SrSv::Agent;
 my $csnick = 'ConnectServ';
 
 agent_connect($csnick, 'services', undef, '+pqzBHS', 'Connection Monitor');
-agent_join($csnick, $config::conf{diag});
-net::setmode($csnick, $config::conf{diag}, '+o', $csnick);
+agent_join($csnick, main_conf_diag);
+ircd::setmode($csnick, main_conf_diag, '+o', $csnick);
 
 addhandler('NICKCONN', undef, undef, 'connectserv::ev_nickconn', 1);
 sub ev_nickconn {
 	my ($nick, $ident, $host, $server, $gecos) = @_[0,3,4,5,9];
-	
+
 	$userlist{lc $nick} = [$ident, $host, $gecos, $server];
-	
-	return unless synced();
+
+	return unless initial_synced();
 	message("\00304\002SIGNED ON\002 user: \002$nick\002 ($ident\@$host - $gecos\017\00304) at $server");
 }
 
@@ -61,7 +70,8 @@ addhandler('CHGIDENT', undef, undef, 'connectserv::ev_identchange', 1);
 sub ev_identchange {
 	my (undef, $nick, $ident) = @_;
 
-	my ($oldident, $host, $gecos, $server) = @{$userlist{lc $nick}} if (defined($userlist{lc $nick}));
+	my ($oldident, $host, $gecos, $server);
+	($oldident, $host, $gecos, $server) = @{$userlist{lc $nick}} if (defined($userlist{lc $nick}));
 	$userlist{lc $nick} = [$ident, $host, $gecos, $server];
 
 	message("\00310\002IDENT CHANGE\002 user: \002$nick\002 ($oldident\@$host) changed their virtual ident to \002$ident\002");
@@ -70,12 +80,13 @@ sub ev_identchange {
 addhandler('QUIT', undef, undef, 'connectserv::ev_quit', 1);
 sub ev_quit {
 	my ($nick, $reason) = @_;
+	return unless synced() && $runlevel == ST_NORMAL;
 	my ($ident, $host, $gecos, $server);
 	if(defined($userlist{lc $nick})) {
 		($ident, $host, $gecos, $server) = @{$userlist{lc $nick}};
 		delete($userlist{lc $nick});
 	}
-	return unless synced();
+	return unless initial_synced();
 	message("\00303\002SIGNED OFF\002 user: \002$nick\002 ($ident\@$host - $gecos\017\00303) at $server - $reason");
 }
 
@@ -113,9 +124,37 @@ sub ev_umode {
 	}
 }
 
+addhandler('SJOIN', undef, undef, 'connectserv::chan_join', 1) if connectserv_conf_joinpart;
+sub chan_join {
+	my ($server, $cn, $ts, $chmodes, $chmodeparms, $userarray, $banarray, $exceptarray) = @_;
+	return unless synced() && $runlevel == ST_NORMAL;
+	foreach my $user (@$userarray) {
+		my $nick = $user->{NICK};
+		message ("\00310CHANNEL JOIN: \002$nick\002 joined to \002$cn\002\003");
+	}
+}
+
+addhandler('PART', undef, undef, 'connectserv::chan_part', 1) if connectserv_conf_joinpart;
+sub chan_part {
+	my ($nick, $cn) = @_;
+	return unless synced() && $runlevel == ST_NORMAL;
+	message ("\00310CHANNEL PART: \002$nick\002 parted from \002$cn\002\003");
+}
+
+addhandler('JOIN', undef, undef, 'connectserv::chan_join0', 1) if connectserv_conf_joinpart;
+sub chan_join0 {
+	my ($nick, $cn) = @_;
+	return unless synced() && $runlevel == ST_NORMAL;
+	if($cn eq '0') {
+		message ("\00310CHANNEL PART: \002$nick\002 parted all channels\003");
+	} else {
+		message ("\00310CHANNEL JOIN: \002$nick\002 joined to \002$cn\002\003");
+	}
+}
+
 sub message(@) {
-	net::privmsg($csnick, $config::conf{'diag'}, @_);
-	log::write_log('diag', '<'.$csnick.'>', @_);
+	ircd::privmsg($csnick, main_conf_diag, @_);
+	write_log('diag', '<'.$csnick.'>', @_);
 }
 
 sub init { }
