@@ -14,48 +14,59 @@
 #	along with SurrealServices; if not, write to the Free Software
 #	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 package core;
+use strict;
 
-use SrSv::Conf 'main';
+#use SrSv::Conf 'main';
+use SrSv::Conf2Consts 'main';
 use SrSv::RunLevel 'main_shutdown';
 use SrSv::IRCd::Event 'addhandler';
 use SrSv::IRCd::IO 'ircsend';
 use SrSv::Timer 'add_timer';
+use SrSv::Time 'time_rel_long_all';
 use SrSv::Agent;
 use SrSv::Process::Init; #FIXME - only needed for ccode
-#FIXME
-#use SrSv::User::Notice;
-sub notice {
-	net::notice('ServServ', shift()->{NICK}, shift);
-}
+use SrSv::User::Notice;
+use SrSv::Help;
+
+my $startTime = time();
 
 our %ccode; #FIXME - Split out
 proc_init {
-	open COUNTRY, "$main::prefix/data/country-codes.txt";
-	while(my $x = <COUNTRY>) {
+	open ((my $COUNTRY), main::PREFIX()."/data/country-codes.txt");
+	while(my $x = <$COUNTRY>) {
 		chomp $x;
 		my($code, $country) = split(/   /, $x);
 		$ccode{uc $code} = $country;
 	}
-	close COUNTRY;
+	close $COUNTRY;
 };
 
 our $rsnick = 'ServServ';
 
-addhandler('PING', undef, undef, 'net::pong', 1);
+addhandler('STATS', undef, undef, 'core::stats');
+sub stats($$) {
+	my ($src, $token) = @_;
+	if($token eq 'u') {
+		ircsend('242 '.$src.' :Server up '.time_rel_long_all($startTime),
+			'219 '.$src.' u :End of /STATS report')
+	}
+}
+
+addhandler('PING', undef, undef, 'ircd::pong', 1);
 
 sub pingtimer($) {
-	net::ping();
+	ircd::ping();
 	add_timer('perlserv__pingtimer', 60, __PACKAGE__, 
 			"core::pingtimer");
 }
 
 agent_connect($rsnick, 'service', undef, '+ABHSNaopqz', 'Services Control Agent');
+agent_join($rsnick, main_conf_diag);
+ircd::setmode($rsnick, main_conf_diag, '+o', $rsnick);
 
 addhandler('SEOS', undef, undef, 'core::ev_connect', 1);
 
 sub ev_connect {
-	agent_join($rsnick, $main_conf{diag});
-	net::setmode($rsnick, $main_conf{diag}, '+o', $rsnick);
 	add_timer('perlserv__pingtimer', 60, __PACKAGE__,
 			"core::pingtimer");
 }
@@ -67,11 +78,11 @@ sub dispatch {
 	my $user = { NICK => $src, AGENT => $rsnick };
 	if(!adminserv::is_ircop($user)) {
 		notice($user, 'Access Denied');
-		net::globops($rsnick, "\002$src\002 failed access to $rsnick $msg");
+		ircd::globops($rsnick, "\002$src\002 failed access to $rsnick $msg");
 		return;
 	}
 	if($msg =~ /^lsmod/i) {
-		notice($user, $main_conf{load});
+		notice($user, main_conf_load);
 	}
 
 	if($msg =~ /^shutdown/i) {
@@ -92,7 +103,7 @@ sub dispatch {
 		ircsend($cmd);
 	}
 	if($msg =~ /^help$/) {
-		help::sendhelp($rsnick, $src, lc 'core');
+		sendhelp($user, lc 'core');
 		return;
 	}
 	if(main::DEBUG and $msg =~ /^eval\s+(.*)/) {
