@@ -22,6 +22,9 @@ use strict;
 use SrSv::MySQL '$dbh';
 use SrSv::Process::Init;
 use SrSv::IRCd::Event 'addhandler';
+use SrSv::IRCd::State 'initial_synced';
+
+use SrSv::Log;
 
 use SrSv::Shared qw(%unwhois);
 
@@ -33,11 +36,6 @@ addhandler('NICKCONN', undef, undef, 'nickconn');
 our ($get_ip_country, $get_ip_country_aton, $get_user_country);
 
 proc_init {
-	# We don't have a good check for this yet.
-	# need explicit module-deps
-	module::is_loaded('services') or
-		die("module services must be loaded first before country::init()!");
-
 	$get_ip_country = $dbh->prepare_cached("SELECT country FROM country WHERE
 		? BETWEEN low AND high");
 	$get_ip_country_aton = $dbh->prepare_cached("SELECT country FROM country WHERE
@@ -98,11 +96,16 @@ sub get_country_long($) {
 	return 'Unknown';
 }
 
+sub get_user_country_long($) {
+	my ($user) = @_;
+	return get_country_long(get_user_country($user));
+}
+
 sub nickconn {
-	my ($rnick, $time, $ident, $host, $vhost, $server, $modes, $gecos, $ip) = @_[0,2..4,8,5,7,9,10];
-	if(net::synced()) {
+	my ($rnick, $time, $ident, $host, $vhost, $server, $modes, $gecos, $ip, $svsstamp) = @_[0,2..4,8,5,7,9,10,6];
+	if(initial_synced() && !$svsstamp) {
 		if ($ip) {
-			log::wlog($main::rsnick, log::INFO(), "\002$rnick\002 is connecting from ".
+			wlog($main::rsnick, LOG_INFO(), "\002$rnick\002 is connecting from ".
 				get_country_long(get_ip_country_aton($ip)));
 		}
 		else {
@@ -122,7 +125,7 @@ sub userip($$$) {
 	return unless($unwhois{lc $nick});
 	return unless($ip =~ /^\d{1,3}(\.\d{1,3}){3}$/);
 
-	log::wlog($main::rsnick, log::INFO(), "\002$nick\002 is connecting from ".
+	wlog($main::rsnick, LOG_INFO(), "\002$nick\002 is connecting from ".
 		get_country_long(get_ip_country_aton($ip)));
 	delete $unwhois{lc $nick};
 }
