@@ -23,9 +23,12 @@ use SrSv::Process::InParent qw(chanlog addchan delchan ev_sjoin ev_join ev_part
 ev_kick ev_mode ev_nickconn ev_nickchange ev_quit ev_message ev_notice
 ev_chghost ev_kill ev_topic ev_connect saveconf loadconf join_chans);
 
+use SrSv::Conf2Consts qw(main);
 use SrSv::IRCd::Event 'addhandler';
-use SrSv::IRCd::State 'synced';
+use SrSv::IRCd::State 'initial_synced';
 use SrSv::Agent;
+use SrSv::User::Notice;
+use SrSv::Log qw( :all );
 
 my %userlist;
 my %chanlist;
@@ -35,19 +38,13 @@ my $chanopmode = '+v';
 
 loadconf();
 agent_connect($lsnick, 'services', undef, '+pqzBHSD', 'Log Service');
-agent_join($lsnick, $config::conf{diag});
-net::setmode($lsnick, $config::conf{diag}, '+o', $lsnick);
+agent_join($lsnick, main_conf_diag);
+ircd::setmode($lsnick, main_conf_diag, '+o', $lsnick);
 join_chans();
 
-sub notice($@) {
-	my $user = shift;
-	
-	net::notice($user->{AGENT} || $lsnick, $user->{NICK}, @_);
-}
-		
 sub chanlog($@) {
 	my ($cn, @payload) = @_;
-	log::write_log("logserv:$cn", '', @payload)
+	write_log("logserv:$cn", '', @payload)
 		# This if allows us to be lazy
 		if defined($chanlist{lc $cn});
 }
@@ -55,10 +52,10 @@ sub chanlog($@) {
 sub addchan($$) {
 	my ($user, $cn) = @_;
 	unless(defined($chanlist{lc $cn})) {
-		log::open_log("logserv:$cn", lc($cn).'.log');
+		open_log("logserv:$cn", lc($cn).'.log');
 		$chanlist{lc $cn} = 1;
 		agent_join($lsnick, $cn);
-		net::setmode($lsnick, $cn, $chanopmode, $lsnick);
+		ircd::setmode($lsnick, $cn, $chanopmode, $lsnick);
 		notice($user, "Channel $cn will now be logged");
 		saveconf();
 		return 1;
@@ -71,7 +68,7 @@ sub addchan($$) {
 sub delchan($$) {
 	my ($user, $cn) = @_;
 	if(defined($chanlist{lc $cn})) {
-		log::close_log("logserv:$cn");
+		close_log("logserv:$cn");
 		delete($chanlist{lc $cn});
 		agent_part($lsnick, $cn, "Channel has been deleted by ".$user->{NICK});
 		notice($user, "Channel $cn will not be logged");
@@ -102,7 +99,7 @@ sub ev_join {
 	{
 		$userlist{lc $nick}{CHANS}{$cn} = 1;
 	}
-	if(synced()) {
+	if(initial_synced()) {
 		if($cn eq '0') {
 			foreach my $cn (keys(%{$userlist{lc $nick}{CHANS}})) {
 				ev_part($nick, $cn, 'Left all channels');
@@ -132,7 +129,7 @@ sub ev_kick {
 	return unless defined($userlist{lc $target}); # Sometimes we get JOINs after a KILL or QUIT
 	if(lc $target eq lc $lsnick) {
 		agent_join($lsnick, $cn);
-		net::setmode($lsnick, $cn, '+o', $lsnick);
+		ircd::setmode($lsnick, $cn, '+o', $lsnick);
 		return;
 	}
 	{
@@ -144,7 +141,7 @@ sub ev_kick {
 addhandler('MODE', undef, undef, 'logserv::ev_mode');
 sub ev_mode {
 	my ($src, $cn, $modes, $parms) = @_;
-	return unless synced();
+	return unless initial_synced();
 	chanlog("$cn", "-!- mode/$cn [$modes".($parms ? " $parms" : '')."] by $src");
 }
 
@@ -157,7 +154,7 @@ sub ev_nickconn {
                                 $vhost = $cloakhost;
                         }
                         else {
-				# Since we have no desire to do net::userhost checks
+				# Since we have no desire to do ircd::userhost checks
 				# This makes us dependent on VHP or CLK.
 				# Do we care? Not at the moment.
 				# This should NEVER happen with VHP or CLK.
@@ -190,7 +187,7 @@ addhandler('QUIT', undef, undef, 'logserv::ev_quit');
 sub ev_quit {
 	my ($nick, $reason) = @_;
 	my ($ident, $vhost) = @{$userlist{lc $nick}{INFO}};
-	if (synced()) {
+	if (initial_synced()) {
 		foreach my $cn (keys(%{$userlist{lc $nick}{CHANS}})) {
 			chanlog($cn, "$nick [$ident\@$vhost] has quit [$reason]");
 		}
@@ -275,7 +272,7 @@ sub ev_kill {
 	my ($src, $target, $reason) = @_;
 	return if is_agent($target) or !defined($userlist{lc $target}); # Ignore agent kills.
 	my ($ident, $vhost) = @{$userlist{lc $target}{INFO}};
-	if (synced()) {
+	if (initial_synced()) {
 		foreach my $cn (keys(%{$userlist{lc $target}{CHANS}})) {
 			chanlog($cn, "$target [$ident\@$vhost] has quit [Killed ($src ($reason))]");
 		}
@@ -287,7 +284,7 @@ addhandler('TOPIC', undef, undef, 'logserv::ev_topic');
 sub ev_topic {
 	my ($src, $cn, $setter, undef, $topic) = @_;
 	# We don't care about the timestamp
-	return unless synced();
+	return unless initial_synced();
 	chanlog($cn, "$src changed the topic of $cn to: $topic".($setter ne $src ? " ($setter)" : ''));
 }
 
@@ -309,9 +306,9 @@ sub loadconf() {
 
 sub join_chans() {
 	foreach my $cn (keys(%chanlist)) {
-		log::open_log("logserv:$cn", lc($cn).'.log');
+		open_log("logserv:$cn", lc($cn).'.log');
 		agent_join($lsnick, $cn);
-		net::setmode($lsnick, $cn, $chanopmode, $lsnick);
+		ircd::setmode($lsnick, $cn, $chanopmode, $lsnick);
 	}
 }
 
