@@ -11,9 +11,7 @@ package securitybot;
 
 use strict;
 no strict "refs";
-use Time::Local;
-#use Time::HiRes qw(sleep gettimeofday);
-use DBI;
+use Time::HiRes qw(gettimeofday);
 
 use SrSv::Process::Init;
 use SrSv::IRCd::Event 'addhandler';
@@ -22,14 +20,23 @@ use SrSv::Timer qw(add_timer);
 use SrSv::Time;
 use SrSv::Agent;
 use SrSv::HostMask qw( parse_hostmask );
-use SrSv::Conf qw(main sql);
+use SrSv::Conf2Consts qw(main sql);
+use SrSv::SimpleHash qw(readHash writeHash);
+
+use SrSv::Log;
+
+use SrSv::User qw( get_user_nick );
+use SrSv::User::Notice;
+use SrSv::Help qw( sendhelp );
 
-use SrSv::MySQL::Glob;
 use SrSv::MySQL '$dbh';
+use SrSv::MySQL::Glob;
 
-use SrSv::Shared qw(%conf %torip %unwhois $last_version);
+use SrSv::Shared qw(%conf %torip %unwhois);
 
-use SrSv::Process::InParent qw( list_conf saveconf loadconf );
+use SrSv::Process::InParent qw(list_conf loadconf saveconf update_tor_list);
+
+use SrSv::TOR;
 
 #this stuff needs to be put into files
 our $sbnick = "SecurityBot";
@@ -91,10 +98,11 @@ proc_init {
 };
 
 sub init {
+	return if main::COMPILE_ONLY();
 	my $tmpdbh = DBI->connect(
-		"DBI:mysql:".$sql_conf{'mysql-db'}.":mysql_server_prepare=1",
-		$sql_conf{'mysql-user'},
-		$sql_conf{'mysql-pass'},
+		"DBI:mysql:".sql_conf_mysql_db,
+		sql_conf_mysql_user,
+		sql_conf_mysql_pass,
 		{
 			AutoCommit => 1,
 			RaiseError => 1
@@ -128,11 +136,11 @@ sub globops($$) {
 }
 
 sub register {
-	agent_connect($sbnick, $ident, $vhost, $umodes, $gecos);
-	net::sqline($sbnick, 'Reserved for Services');
+	agent_connect($sbnick, $ident, undef, $umodes, $gecos);
+	ircd::sqline($sbnick, 'Reserved for Services');
 	
-	agent_join($sbnick, $main_conf{diag});
-	net::setmode($sbnick, $main_conf{diag}, '+o', $sbnick);
+	agent_join($sbnick, main_conf_diag);
+	ircd::setmode($sbnick, main_conf_diag, '+o', $sbnick);
 }
 
 sub start_timers {
@@ -142,7 +150,7 @@ sub start_timers {
 
 sub start_timers2 {
 	update_tor_list_timed(3540) if $conf{'EnableTor'};
-	update_ss_timed(3300) if $conf{'EnableSS'};
+	#securitybot::ss2tkl::update_ss_timed(3300) if $conf{'EnableSS'};
 };
 
 sub nickconn {
@@ -155,7 +163,7 @@ sub nickconn {
 			check_blacklists($rnick, $ip) or return;
 		}
 		else {
-			net::userip($rnick) unless module::is_loaded('services');
+			ircd::userip($rnick) unless module::is_loaded('services');
 		}
 	}
 
@@ -163,11 +171,10 @@ sub nickconn {
 		my @ctcplist = split(/ /, $conf{'CTCPonConnect'});
 		foreach my $ctcp_msg (@ctcplist) {
 			if(uc($ctcp_msg) eq 'PING') {
-				# This really should use Time::HiRes::gettimeofday
-				# and the last param should be the usec portion
-				net::ctcp($sbnick, $rnick, 'PING', time(), 0);
+				my ($sec, $usec) = gettimeofday();
+				ircd::ctcp($sbnick, $rnick, 'PING', $sec, $usec);
 			} else {
-				net::ctcp($sbnick, $rnick, uc($ctcp_msg));
+				ircd::ctcp($sbnick, $rnick, uc($ctcp_msg));
 			}
 		}
 	}
@@ -175,8 +182,6 @@ sub nickconn {
 	$unwhois{lc $rnick} = 1 unless ($svsstamp or $ip);
 }
 
-# XXX - we depend on nickserv to send userip to every user, so this won't work
-# without services loaded
 sub userip {
 	my($src, $rnick, $ip) = @_;
 
@@ -198,7 +203,12 @@ sub check_opm($) {
 
 sub check_country($) {
 	my ($ip) = @_;
-	my $ccode = country::get_ip_country_aton($ip);
+	my $ccode; 
+	if(module::is_loaded('geoip')) {
+		$ccode = geoip::get_ip_location($ip); 
+	} elsif(module::is_loaded('country')) {
+		$ccode = country::get_ip_country_aton($ip);
+	}
 	foreach my $country (split(/[, ]+/, $conf{'BanCountry'})) {
 		if (lc $ccode eq lc $country) {
 			return country::get_country_long($country);
@@ -218,20 +228,24 @@ sub check_blacklists($$) {
 	
 	if(initial_synced and $enabletor && $torip{$ip}) {
 		if (lc $enabletor eq lc 'vhost') {
-			net::chghost($sbnick, $rnick, misc::gen_uuid(1, 20)).'.session.tor';
+			ircd::chghost($sbnick, $rnick, misc::gen_uuid(1, 20).'.session.tor');
 		} else {
-			net::zline($sbnick, $ip, $conf{'ProxyZlineTime'}, $conf{'TorZlineReason'});
+			ircd::zline($sbnick, $ip, $conf{'ProxyZlineTime'}, $conf{'TorZlineReason'});
 		}
 		return 0;
 	}
 
 	if($conf{'EnableOPM'} && check_opm($ip)) {
-		net::zline($sbnick, $ip, $conf{'ProxyZlineTime'}, mk_banreason($conf{'OPMZlineReason'}, $ip));
+		ircd::zline($sbnick, $ip, $conf{'ProxyZlineTime'}, mk_banreason($conf{'OPMZlineReason'}, $ip));
 		return 0;
 	}
 
-	if($conf{'BanCountry'} && module::is_loaded('country') && (my $country = check_country($ip))) {
-		net::zline($sbnick, $ip, $conf{'ProxyZlineTime'}, mk_banreason($conf{'CountryZlineReason'}, $country));
+sub hasGeoCountry() {
+	return module::is_loaded('country') || module::is_loaded('geoip');
+}
+
+	if($conf{'BanCountry'} && hasGeoCountry() && (my $country = check_country($ip))) {
+		ircd::zline($sbnick, $ip, $conf{'ProxyZlineTime'}, mk_banreason($conf{'CountryZlineReason'}, $country));
 		return 0;
 	}
 
@@ -248,25 +262,16 @@ sub update_tor_list_timed($) {
 }
 
 sub update_tor_list() {
+	return unless (defined($conf{'TorServer'}) && length($conf{'TorServer'}));
 	diagmsg( " -- Loading Tor server list.");
 	
 	# path may be a local one if you run a tor-client.
 	# most configs are /var/lib/tor/cached-directory
-	if ($conf{'TorServer'} =~ /^(http|ftp)\:\/\//i ) {
-		open TOR, '-|', ('wget -q -O - ' . $conf{'TorServer'});
-	} else {
-		open TOR, '<', $conf{'TorServer'};
-	}
-
 	my %newtorip;
-	while(my $l = <TOR>) {
-		if($l =~ /^router (\S+) (\S+)/) {
-			$newtorip{$2} = 1 unless $2 =~ /^(127|10|192\.168)\./;
-		}
+	foreach my $torIP (getTorRouters($conf{'TorServer'})) {
+		$newtorip{$torIP} = 1;
 	}
 
-	close TOR;
-
 	my $torcount = scalar(keys(%newtorip));
 
 	if($torcount > 0) {
@@ -282,22 +287,21 @@ sub msghandle {
 		print join("\n", @_);
 		my $user = { NICK => $rnick, AGENT => $sbnick };
 		unless (adminserv::is_ircop($user)) {
-			net::notice($sbnick, $rnick, 'Permission Denied');
+			notice($user, 'Permission Denied');
 			return;
 		}
 
 		if($msg =~ /^help/i) {
-			my @args = split(/ /, $msg);
-			shift @args; #discard first token 'help'
-			help::sendhelp($sbnick, $rnick, lc $sbnick, @args);
+			my (undef, @args) = split(/ /, $msg); #discards first token 'help'
+			sendhelp($user, 'securitybot', @args);
 		}
 
 		elsif($msg =~ /^notice (\S*) (.*)/i) {
-			net::notice($sbnick, $1, $2);
+			ircd::notice($sbnick, $1, $2);
 		}
 
 		elsif($msg =~ /^msg (\S*) (.*)/i) {
-			net::privmsg($sbnick, $1, $2);
+			ircd::privmsg($sbnick, $1, $2);
 		}
 
 		elsif($msg =~ /^raw (.*)/i) {
@@ -305,15 +309,15 @@ sub msghandle {
 				notice($user, 'You do not have sufficient rank for this command');
 				return;
 			}
-			net::ircsend($1);
+			ircd::ircsend($1);
 		}
 
 		elsif($msg =~ /^kill (\S*) (.*)/i) {
-			net::irckill($sbnick, $1, $2);
+			ircd::irckill($sbnick, $1, $2);
 		}
 
 		elsif($msg =~ /^conf/i) {
-			net::notice($sbnick, $rnick, "Configuration:", list_conf);
+			notice($user, "Configuration:", list_conf);
 		}
 
 		elsif($msg =~ /^set (\S+) (.*)/i) {
@@ -326,26 +330,26 @@ sub msghandle {
 			chomp $p[1];
 
 			if(update_conf($p[0], $p[1])) {
-				net::notice($sbnick, $rnick, "Configuration: ".$p[0]." = ".$p[1]);
+				notice($user, "Configuration: ".$p[0]." = ".$p[1]);
 			} else {
-				net::notice($sbnick, $rnick, "That value is read-only.");
+				notice($user, "That value is read-only.");
 			}
 		}
 
 		elsif($msg =~ /^save/i) {
-			net::notice($sbnick, $rnick, "Saving configuration.");
+			notice($user, "Saving configuration.");
 
 			saveconf();
 		}
 
 		elsif($msg =~ /^rehash/i) {
-			net::notice($sbnick, $rnick, "Loading configuration.");
+			notice($user, "Loading configuration.");
 
 			loadconf(1);
 		}
 
 		elsif($msg =~ /^tssync/i) {
-			net::tssync();
+			ircd::tssync();
 		}
 
 		elsif($msg =~ /^svsnick (\S+) (\S+)/i) {
@@ -353,21 +357,20 @@ sub msghandle {
 				notice($user, 'You do not have sufficient rank for this command');
 				return;
 			}
-			net::svsnick($sbnick, $1, $2);
+			ircd::svsnick($sbnick, $1, $2);
 		}
 
 		elsif($msg =~ /^tor-update/i) {
-			net::notice($sbnick, $rnick, "Updating Tor server list.");
+			notice($user, "Updating Tor server list.");
 			update_tor_list();
 		}
+=cut
 		elsif($msg =~ /^ss-update/i) {
-			net::notice($sbnick, $rnick, "Updating SS definitions.");
-			update_ss();
+			notice($user, "Updating SS definitions.");
+			securitybot::ss2tkl::update_ss();
 		}
-
+=cut
 		elsif($msg =~ /^tkl/i) {
-			#$msg =~ s/^tkl list//i;
-			#my @args = split(/ /, $msg);
 			sb_tkl($user, $msg);
 		}
 }
@@ -396,7 +399,8 @@ sub sb_tkl($$) {
 # Better fix is to rewrite msghandle in another cleanup patch.
 	my ($user, $msg) = @_;
 	# We discard first token 'tkl'
-	my (undef, $cmd, $msg) = split(/ /, $msg, 3);
+	my $cmd;
+	(undef, $cmd, $msg) = split(/ /, $msg, 3);
 	if(lc($cmd) eq 'list') {
 		if($msg) {
 			sb_tkl_glob($user, $msg);
@@ -407,7 +411,7 @@ sub sb_tkl($$) {
 	}
 	elsif(lc($cmd) eq 'del') {
 		unless($msg) {
-			nickserv::notice($user, "You have to specify at least one parameter");
+			notice($user, "You have to specify at least one parameter");
 		}
 		sb_tkl_glob_delete($user, $msg);
 	}
@@ -432,7 +436,7 @@ sub sb_tkl_list($) {
 	$get_all_tklban->finish();
 	push @reply, "No results" unless @reply;
 
-	nickserv::notice($user, @reply);
+	notice($user, @reply);
 }
 
 sub sb_tkl_glob($$) {
@@ -443,14 +447,14 @@ sub sb_tkl_glob($$) {
 	my ($filters, $parms) = split(/ /, $cmdline, 2);
 	my @filters = split(//, $filters);
 	unless($filters[0] eq '+' or $filters[0] eq '-') {
-		nickserv::notice($user, "Invalid Syntax: First parameter must be a set of filters preceded by a + or -");
+		notice($user, "Invalid Syntax: First parameter must be a set of filters preceded by a + or -");
 		return;
 	}
 	my @args = misc::parse_quoted($parms);
 
 	my ($success, $expr) = make_tkl_query(\@filters, \@args);
 	unless ($success) {
-		nickserv::notice($user, "Error: $expr");
+		notice($user, "Error: $expr");
 		return;
 	}
 	$sql_expr .= $expr;
@@ -473,7 +477,7 @@ sub sb_tkl_glob($$) {
 	$get_glob_tklban->finish();
 
 	push @reply, "No results" unless @reply;
-	nickserv::notice($user, @reply);
+	notice($user, @reply);
 }
 
 sub sb_tkl_glob_delete($$) {
@@ -484,28 +488,28 @@ sub sb_tkl_glob_delete($$) {
 	my ($filters, $parms) = split(/ /, $cmdline, 2);
 	my @filters = split(//, $filters);
 	unless($filters[0] eq '+' or $filters[0] eq '-') {
-		nickserv::notice($user, "Invalid Syntax: First parameter must be a set of filters preceded by a + or -");
+		notice($user, "Invalid Syntax: First parameter must be a set of filters preceded by a + or -");
 		return;
 	}
 	my @args = misc::parse_quoted($parms);
 
 	my ($success, $expr) = make_tkl_query(\@filters, \@args);
 	unless ($success) {
-		nickserv::notice($user, "Error: $expr");
+		notice($user, "Error: $expr");
 		return;
 	}
 
 	$sql_expr .= $expr;
 
-	my $src = $user->{ NICK }; # the nickserv function is kinda overkill here.
+	my $src = get_user_nick($user);
 	my $get_glob_tklban = $dbh->prepare($sql_expr);
 	$get_glob_tklban->execute();
 	while(my ($type, $ident, $host) = $get_glob_tklban->fetchrow_array()) {
 		if($type eq 'G') {
-			net::unkline($src, $ident, $host);
+			ircd::unkline($src, $ident, $host);
 		}
 		elsif($type eq 'Z') {
-			net::unzline($src, $host);
+			ircd::unzline($src, $host);
 		}
 	}
 	$get_glob_tklban->finish();
@@ -640,12 +644,12 @@ sub handle_tkl($$@) {
 			diagmsg( get_tkl_type_name($type)." added for $ident\@$host ".
 				"from ($setter on ".gmtime2($time).
 				($expire ? ' to expire at '.gmtime2($expire) : ' does not expire').": $reason)")
-					if net::synced() and $type ne 'Q';
+					if initial_synced() and $type ne 'Q';
 		}
 		elsif($sign == -1) {
 			my ($ident, $host, $setter) = @parms;
 
-			if ($type ne 'Q' and net::synced()) {
+			if ($type ne 'Q' and initial_synced()) {
 				$get_tklban->execute($type, $ident, $host);
 				my (undef, $expire, $time, $reason) = $get_tklban->fetchrow_array;
 				$get_tklban->finish();
@@ -665,12 +669,12 @@ sub handle_tkl($$@) {
 			$add_spamfilter->finish();
 			diagmsg( "Spamfilter added: '$mask' [target: $target] [action: ".
 				get_filter_action_name($action)."] [reason: $reason] on ".gmtime2($time)."from ($setter)")
-					if net::synced();
+					if initial_synced();
 		}
 		elsif($sign == -1) {
 			# TKL - F u Z tabris!northman@tabris.netadmin.SCnet.ops 0 0 :do_not!use@mask
 			my ($target, $action, $setter, $mask) = @parms;
-			if(net::synced()) {
+			if(initial_synced()) {
 				$get_spamfilter->execute($target, $action, $mask);
 				my ($time, $reason) = $get_spamfilter->fetchrow_array;
 				$get_spamfilter->finish();
@@ -685,13 +689,13 @@ sub handle_tkl($$@) {
 }
 
 sub saveconf() {
-	config::writeHash(\%conf, "config/securitybot/sb.conf");
+	writeHash(\%conf, "config/securitybot/sb.conf");
 }
 
 sub loadconf($) {
 	my ($update) = @_;
 	
-	%conf = config::readHash("config/securitybot/sb.conf");
+	%conf = readHash("config/securitybot/sb.conf");
 }
 
 sub update_conf($$) {
@@ -703,150 +707,6 @@ sub update_conf($$) {
 	return 1;
 }
 
-########################################################################
-#                       SecureServ definitions                         #
-########################################################################
-my $ss_base_url = 'http://secure.irc-chat.net/defs.php';
-
-sub get_ss_definitions() {
-	my $url = "'$ss_base_url?u=".$conf{'ss_Username'}.'&p='.$conf{'ss_Pass'}."'";
-
-	my %skipfilters;
-	foreach my $skipfilter (split(/ /, $conf{'ss_SkipFilters'})) {
-		$skipfilters{uc($skipfilter)} = 1;
-	}
-	
-	open DATAFILE, '-|', ('wget -q -O - ' . $url);
-	#open DATAFILE, "$main::prefix/data/viri.dat";
-
-	my (@filters, $version);
-	while (my $l = <DATAFILE>) {
-	#XcoderBot 2 0 0 "^Xcoder.*" "You appear to be infected with a BotNet Trojan" 1
-		chomp $l;
-		if($l =~ /^(\S+) (\d) (\d) (\d) \"(.+)\" \"(.+)\" (\d)$/) {
-			my ($filter, $type, $regex, $reason, $action) = ($1,$2,$5,$6,$7);
-			next if $skipfilters{uc($filter)};
-			my $ref = make_ss_tkl($filter, $type, $regex, $reason, $action);
-			push @filters, $ref if $ref;
-		}
-		if ($l =~ /^(\d+)$/) {
-			$version = $1;
-		}
-	}
-	return ($version, @filters);
-}
-
-sub make_ss_tkl($$$$$) {
-	my ($filter, $type, $regex, $reason, $action) = @_;
-	my ($tkl_target, $tkl_action);
-	$regex =~ s/^\s+//;
-	$reason =~ s/ /_/g;
-	if($regex =~ /\(\?\i\)/) {
-		$regex =~ s/\(\?\i\)//;
-	}
-	else {
-		$regex = "(?i)$regex";
-	}
-	if($type == 0) {
-		if ($regex =~ /^\^/ or $regex =~ /\$$/) {
-			$regex =~ s/^\^/\^\x01/;
-			$regex =~ s/\$$/\x01\$/;
-		}
-		else {
-			$regex = "\x01$regex\x01";
-		}
-		$tkl_target = 'n';
-	}
-	elsif($type == 1 or $type == 6) {
-		$tkl_target = 'cpnNPq';
-	}
-	elsif($type == 2) {
-		$tkl_target = 'u';
-		$regex =~ s/\$$//;
-		$regex = "$regex!";
-	}
-	elsif($type == 3) {
-		$tkl_target = 'u';
-		$regex =~ s/\$$//;
-		$regex =~ s/^\^//;
-		$regex = "!$regex@";
-	}
-	elsif($type == 4) {
-		$tkl_target = 'u';
-		$regex =~ s/^\^//;
-		$regex = ":$regex";
-	}
-	elsif($type == 5) {
-		# We don't have a spamfilter type for this
-		# this would be a drone channel.
-		return undef;
-	}
-
-	if($action == 0) {
-		$tkl_action = 'v'; # viruschan
-	}
-	elsif($action == 1) {
-		$tkl_action = 'Z'; # gzline. gline would be better emulation
-	}
-	elsif($action == 2 or $action == 3) {
-		$tkl_action = 'b'; # block.
-		# would be better to trap but not block, but there is no such spamfilter action.
-	}
-
-	return [$tkl_target, $tkl_action, "$filter:_$reason", $regex];
-}
-
-########################################################################
-#   END                 SecureServ definitions                   END   #
-########################################################################
-
-sub update_ss_timed($) {
-	my ($time) = @_;
-
-	add_timer('3600', $time, __PACKAGE__, "securitybot::update_ss_timed");
-
-	update_ss();
-}
-
-sub update_ss() {
-	my %ss_filters;
-	diagmsg( " -- Loading SecureServ definitions.");
-	$get_all_spamfilter->execute();
-	while(my ($target, $action, $setter, $expire, $time, $bantime, $reason, $regex) =
-		$get_all_spamfilter->fetchrow_array())
-	{
-		next unless $setter eq $sbnick;
-		$ss_filters{"$target $action $reason $regex"} = [$target, $action, $reason, $regex];
-	}
-	$get_all_spamfilter->finish();
-
-	my ($version, @filters) = get_ss_definitions();
-	if ($last_version and $version <= $last_version) {
-		diagmsg( " -- SecureServ definitions already up-to-date.");
-		return;
-	}
-	$last_version = $version;
-	unless (@filters) {
-		diagmsg( " -- Loading SecureServ definitions failed.");
-		return;
-	}
-
-	foreach my $tmp (@filters) {
-		my ($target, $action, $reason, $regex) = @$tmp;
-		if(!defined($ss_filters{"$target $action $reason $regex"})) {
-			net::spamfilter(+1, $target, $action, $sbnick, $conf{'DroneKlineTime'}, $reason, $regex);
-		}
-		delete($ss_filters{"$target $action $reason $regex"});
-	}
-
-	foreach my $filter_str (keys(%ss_filters)) {
-		my ($target, $action, $reason, $regex) = @{$ss_filters{$filter_str}};
-		net::spamfilter(0, $target, $action, $sbnick, $conf{'DroneKlineTime'}, $reason, $regex);
-	}
-	diagmsg( " -- Loading SecureServ definitions succeeded.");
-	
-}
-
 sub expire_tkl() {
 	$get_expired_tklban->execute();
 	while (my ($type, $ident, $host, $setter, $expire, $time, $reason) = $get_expired_tklban->fetchrow_array()) {
@@ -873,8 +733,8 @@ sub expire_tkl_timed {
 }
 
 sub diagmsg(@) {
-	net::privmsg($sbnick, $main_conf{diag}, @_);
-	log::write_log('diag', '<'.$main_conf{local}.'>', @_);
+	ircd::privmsg($sbnick, main_conf_diag, @_);
+	write_log('diag', '<'.main_conf_local.'>', @_);
 }
 
 sub end { }
