@@ -16,12 +16,16 @@
 package services;
 use strict;
 
-use DBI;
+use SrSv::Conf::services;
+
 use SrSv::MySQL qw($dbh);
 use SrSv::Conf qw(main services sql);
+use SrSv::Conf2Consts qw(main services sql);
 use SrSv::Timer qw(add_timer);
 use SrSv::Agent;
 use SrSv::IRCd::Event qw(addhandler);
+use SrSv::Log;
+use SrSv::User qw( :flood __flood_expire );
 
 use modules::serviceslibs::adminserv;
 use modules::serviceslibs::nickserv;
@@ -31,24 +35,53 @@ use modules::serviceslibs::botserv;
 use modules::serviceslibs::memoserv;
 use modules::serviceslibs::hostserv;
 
-*conf = \%services_conf;
+*conf = \%services_conf; # only used in some help docs
 
 our @agents = (
-	[$nickserv::nsnick, '+opqzBHS', 'Nick Registration Agent'],
-	[$chanserv::csnick, '+pqzBS', 'Channel Registration Agent'],
-	[$operserv::osnick, '+opqzBHS', 'Operator Services Agent'],
-	[$memoserv::msnick, '+pqzBS', 'Memo Exchange Agent'],
-	[$botserv::bsnick, '+pqzBS', 'Channel Bot Control Agent'],
-	[$adminserv::asnick, '+pqzBS', 'Services\' Administration Agent'],
-	[$hostserv::hsnick, '+pqzBs', 'vHost Agent']
+	[$nickserv::nsnick_default, '+opqzBHS', 'Nick Registration Agent'],
+	[$chanserv::csnick_default, '+pqzBS', 'Channel Registration Agent'],
+	[$operserv::osnick_default, '+opqzBHS', 'Operator Services Agent'],
+	[$memoserv::msnick_default, '+pqzBS', 'Memo Exchange Agent'],
+	[$botserv::bsnick_default, '+pqzBS', 'Channel Bot Control Agent'],
+	[$adminserv::asnick_default, '+pqzBS', 'Services\' Administration Agent'],
+	[$hostserv::hsnick_default, '+pqzBS', 'vHost Agent']
 );
+if(services_conf_nickserv) {
+	push @agents, [services_conf_nickserv, '+opqzBHS', 'Nick Registration Agent'];
+	$nickserv::nsnick = services_conf_nickserv;
+}
+if(services_conf_chanserv) {
+	push @agents, [services_conf_chanserv, '+pqzBS', 'Channel Registration Agent'];
+	$chanserv::csnick = services_conf_chanserv;
+}
+if(services_conf_operserv) {
+	push @agents, [services_conf_operserv, '+opqzBHS', 'Operator Services Agent'];
+	$operserv::osnick = services_conf_operserv;
+}
+if(services_conf_memoserv) {
+	push @agents, [services_conf_memoserv, '+pqzBS', 'Memo Exchange Agent'];
+	$memoserv::msnick = services_conf_memoserv;
+}
+if(services_conf_botserv) {
+	push @agents, [services_conf_botserv, '+pqzBS', 'Channel Bot Control Agent'];
+	$botserv::bsnick = services_conf_botserv;
+}
+if(services_conf_adminserv) {
+	push @agents, [services_conf_adminserv, '+pqzBS', 'Services\' Administration Agent'];
+	$adminserv::asnick = services_conf_adminserv;
+}
+if(services_conf_hostserv) {
+	push @agents, [services_conf_hostserv, '+pqzBS', 'vHost Agent'];
+	$hostserv::hsnick = services_conf_hostserv;
+}
+
 our $qlreason = 'Reserved for Services';
 
 foreach my $a (@agents) {
 	agent_connect($a->[0], 'services', undef, $a->[1], $a->[2]);
-	net::sqline($a->[0], $qlreason);
-	agent_join($a->[0], $main_conf{diag});
-	net::setmode($main::rsnick, $main_conf{diag}, '+o', $a->[0]);
+	ircd::sqline($a->[0], $qlreason);
+	agent_join($a->[0], main_conf_diag);
+	ircd::setmode($main::rsnick, main_conf_diag, '+o', $a->[0]);
 }
 
 addhandler('SEOS', undef, undef, 'services::ev_connect');
@@ -79,9 +112,10 @@ addhandler('UMODE', undef, undef, 'nickserv::umode');
 addhandler('CHGHOST', undef, undef, 'nickserv::chghost');
 addhandler('CHGIDENT', undef, undef, 'nickserv::chgident');
 addhandler('USERIP', undef, undef, 'nickserv::userip');
-addhandler('SQUIT', undef, undef, 'nickserv::squit') if net::NOQUIT();
+addhandler('SQUIT', undef, undef, 'nickserv::squit') if ircd::NOQUIT();
 
 addhandler('PRIVMSG', undef, 'nickserv', 'nickserv::dispatch');
+addhandler('PRIVMSG', undef, lc services_conf_nickserv, 'nickserv::dispatch') if services_conf_nickserv;
 
 addhandler('BACK', undef, undef, 'nickserv::notify_auths');
 
@@ -94,22 +128,32 @@ addhandler('MODE', undef, qr/^#/, 'chanserv::chan_mode');
 addhandler('TOPIC', undef, undef, 'chanserv::chan_topic');
 
 addhandler('PRIVMSG', undef, 'chanserv', 'chanserv::dispatch');
+addhandler('PRIVMSG', undef, lc services_conf_chanserv, 'chanserv::dispatch') if services_conf_chanserv;
 
 # OperServ
 addhandler('PRIVMSG', undef, 'operserv', 'operserv::dispatch');
+addhandler('PRIVMSG', undef, lc services_conf_operserv, 'operserv::dispatch') if services_conf_operserv;
 
-add_timer('flood_expire', 10, __PACKAGE__, 'operserv::flood_expire');
+add_timer('flood_expire', 10, __PACKAGE__, 'services::flood_expire');
+
+sub flood_expire(;$) {
+	add_timer('flood_expire', 10, __PACKAGE__, 'services::flood_expire');
+	__flood_expire();
+}
 
 # MemoServ
 addhandler('PRIVMSG', undef, 'memoserv', 'memoserv::dispatch');
+addhandler('PRIVMSG', undef, lc services_conf_memoserv, 'memoserv::dispatch') if services_conf_memoserv;
 addhandler('BACK', undef, undef, 'memoserv::notify');
 
 # BotServ
 addhandler('PRIVMSG', undef, undef, 'botserv::dispatch');
+# botserv takes all PRIVMSG and NOTICEs, so no special dispatch is needed.
 addhandler('NOTICE', undef, qr/^#/, 'botserv::chan_msg');
 
 # AdminServ
 addhandler('PRIVMSG', undef, 'adminserv', 'adminserv::dispatch');
+addhandler('PRIVMSG', undef, lc services_conf_adminserv, 'adminserv::dispatch') if services_conf_adminserv;
 
 add_timer('', 30, __PACKAGE__, 'services::maint');
 #add_timer('', 20, __PACKAGE__, 'nickserv::cleanup_users');
@@ -117,6 +161,7 @@ add_timer('', 60, __PACKAGE__, 'nickserv::expire_silence_timed');
 
 # HostServ
 addhandler('PRIVMSG', undef, 'hostserv', 'hostserv::dispatch');
+addhandler('PRIVMSG', undef, lc services_conf_hostserv, 'hostserv::dispatch') if services_conf_hostserv;
 
 # $nick should be a registered root nick, if applicable
 # $src is the nick or nickid that sent the command
@@ -127,21 +172,22 @@ sub ulog($$$$;$$) {
 
 	# TODO - Record this in the database
 	
-	log::wlog($service, $level, "$hostmask - $text");
+	wlog($service, $level, "$hostmask - $text");
 }
 
 sub maint {
-	log::wlog($main::rsnick, log::INFO(), " -- Running maintenance routines.");
+	wlog($main::rsnick, LOG_INFO(), " -- Running maintenance routines.");
 	add_timer('', 3600, __PACKAGE__, 'services::maint');
 
 	nickserv::expire();
 	chanserv::expire();
 
-	log::wlog($main::rsnick, log::INFO(), " -- Maintenance routines complete.");
+	wlog($main::rsnick, LOG_INFO(), " -- Maintenance routines complete.");
 }
 
 sub init {
-	my $tmpdbh = DBI->connect("DBI:mysql:".$sql_conf{'mysql-db'}, $sql_conf{'mysql-user'}, $sql_conf{'mysql-pass'}, {  AutoCommit => 1, RaiseError => 1 });
+	return if main::COMPILE_ONLY();
+	my $tmpdbh = DBI->connect("DBI:mysql:".sql_conf_mysql_db, sql_conf_mysql_user, sql_conf_mysql_pass, {  AutoCommit => 1, RaiseError => 1 });
 
 	$tmpdbh->do("TRUNCATE TABLE chanuser");
 	$tmpdbh->do("TRUNCATE TABLE nickchg");
@@ -149,22 +195,6 @@ sub init {
 	$tmpdbh->do("TRUNCATE TABLE chanban");
 	$tmpdbh->do("UPDATE user SET online=0, quittime=".time());
 
-	#$tmpdbh->do("DROP TABLE IF EXISTS olduser");
-	#$tmpdbh->do("CREATE TABLE `newuser` (
-	#	`id` int(10) unsigned NOT NULL default '0',
-	#	`nick` char(30) NOT NULL default '',
-	#	`time` int(10) unsigned NOT NULL default '0',
-	#	`inval` tinyint(4) NOT NULL default '0',
-	#	`ident` char(10) NOT NULL default '',
-	#	`host` char(64) NOT NULL default '',
-	#	`vhost` char(64) NOT NULL default '',
-	#	`server` char(64) NOT NULL default '',
-	#	`modes` char(30) NOT NULL default '',
-	#	`gecos` char(50) NOT NULL default '',
-	#	`guest` tinyint(4) NOT NULL default '0',
-	#	PRIMARY KEY  (`id`), UNIQUE KEY `nick` (`nick`)) TYPE=HEAP");
-	#$dbh->do("RENAME TABLE user TO olduser, newuser TO user");
-
 	$tmpdbh->disconnect;
 }
 
