@@ -23,6 +23,10 @@ use SrSv::Text::Format qw(columnar);
 use SrSv::Errors;
 
 use SrSv::User qw(get_user_nick get_user_id);
+use SrSv::User::Notice;
+use SrSv::Help qw( sendhelp );
+
+use SrSv::Log;
 
 use SrSv::NickReg::Flags qw(NRF_NOHIGHLIGHT nr_chk_flag_user);
 
@@ -35,8 +39,9 @@ use constant {
 	S_ROOT => 4,
 };
 
-*notice = \&nickserv::notice;
+our (%flags, @levels, @defflags, $allflags);
 
+BEGIN {
 # BE CAREFUL CHANGING THESE
 my @flags = (
 	'SERVOP',
@@ -49,23 +54,27 @@ my @flags = (
 	'KILL',
 	'HELP',
 );
-my $allflags = (2**@flags) - 1;
-my %flags;
-for(my $i=0; $i<@flags; $i++) {
-	$flags{$flags[$i]} = 2**$i;
+
+for(my $i = scalar(@flags) - 1; $i >= 0; $i--) {
+	$flags{$flags[$i]} = 1 << $i;
 }
-
-our $asnick = 'AdminServ';
-
+$allflags = (1 << scalar(@flags)) - 1;
 our @levels = ('Normal User', 'HelpOp', 'Operator', 'Administrator', 'Root');
-my @defflags = (
+# BE CAREFUL CHANGING THESE
+our @defflags = (
 	0, # Unused
 	$flags{HELP}, # HelpOp
 	$flags{HELP}|$flags{FJOIN}|$flags{QLINE}|$flags{SUPER}|$flags{FREEZE}|$flags{KILL}, # Operator
-	$flags{HELP}|$flags{FJOIN}|$flags{QLINE}|$flags{SUPER}|$flags{FREEZE}|$flags{HOLD}|$flags{BOT}|$flags{SERVOP}|$flags{KILL}, # Admin
+	$flags{HELP}|$flags{FJOIN}|$flags{QLINE}|$flags{SUPER}|$flags{FREEZE}|$flags{KILL}|
+		$flags{HOLD}|$flags{BOT}|$flags{SERVOP}, # Admin
 	$allflags # Root
 );
 
+}
+our $asnick_default = 'AdminServ';
+our $asnick = $asnick_default;
+
+
 our (
 	$create_svsop, $delete_svsop, $rename_svsop,
 
@@ -100,13 +109,13 @@ sub dispatch($$$) {
 	my @args = split(/\s+/, $msg);
 	my $cmd = shift @args;
 
-	my $user = { NICK => $src, AGENT => $asnick };
+	my $user = { NICK => $src, AGENT => $dst };
 
-	services::ulog($asnick, log::INFO(), "cmd: [$msg]", $user);
+	services::ulog($asnick, LOG_INFO(), "cmd: [$msg]", $user);
 
 	unless(is_svsop($user) or is_ircop($user)) {
 		notice($user, $err_deny);
-		net::globops($asnick, "\002$src\002 failed access to $asnick $msg");
+		ircd::globops($asnick, "\002$src\002 failed access to $asnick $msg");
 		return;
 	}
 
@@ -146,7 +155,7 @@ sub dispatch($$$) {
 		}
 	}
 	elsif($cmd =~ /^help$/i) {
-		help::sendhelp($asnick, $src, 'adminserv', @args)
+		sendhelp($user, 'adminserv', @args)
 	}
 	elsif($cmd =~ /^staff$/i) {
 		if(@args == 0) {
@@ -177,7 +186,7 @@ sub as_svs_add($$$) {
 	$set_svs_level->execute($level, $oper, $root);
 	
 	notice($user, "\002$nick\002 is now a \002Services $levels[$level]\002.");
-	log::wlog($asnick, log::INFO(), "$src added $root as a Services $levels[$level].");
+	wlog($asnick, LOG_INFO(), "$src added $root as a Services $levels[$level].");
 }
 
 sub as_svs_del($$) {
@@ -194,7 +203,7 @@ sub as_svs_del($$) {
 	
 	$delete_svsop->execute($root);
 	notice($user, "\002$nick\002 has been stripped of services rank.");
-	log::wlog($asnick, log::INFO(), "$src stripped $root of services rank.")
+	wlog($asnick, LOG_INFO(), "$src stripped $root of services rank.")
 }
 
 sub as_svs_list($$) {
@@ -288,7 +297,8 @@ sub is_ircop($) {
 
 	my %umodes = modes::splitumodes(nickserv::get_user_modes($user));
 
-	if(defined(%umodes) and (($umodes{'o'} eq '+') or ($umodes{'S'} eq '+'))) {
+	no warnings 'deprecated';
+	if(($umodes{'o'} eq '+') or ($umodes{'S'} eq '+')) {
 		$user->{IRCOP} = 1;
 	}
 	else {
@@ -309,7 +319,7 @@ sub is_service($) {
 
 	my %umodes = modes::splitumodes(nickserv::get_user_modes($user));
 
-	if(defined(%umodes) and ($umodes{'S'} eq '+')) {
+	if($umodes{'S'} eq '+') {
 		$user->{SERVICE} = 1;
 		$user->{IRCOP} = 1;
 	}
