@@ -18,13 +18,19 @@ package botserv;
 use strict;
 no strict 'refs';
 
+use Safe;
+
 use SrSv::Agent;
 use SrSv::Process::Worker 'ima_worker'; #FIXME
 
 use SrSv::Text::Format qw(columnar);
 use SrSv::Errors;
 
-use SrSv::User qw(get_user_nick get_user_id);
+use SrSv::Conf2Consts qw( main services );
+
+use SrSv::User qw(get_user_nick get_user_id :flood);
+use SrSv::User::Notice;
+use SrSv::Help qw( sendhelp );
 
 use SrSv::ChanReg::Flags;
 use SrSv::NickReg::Flags qw(NRF_NOHIGHLIGHT nr_chk_flag_user);
@@ -36,17 +42,19 @@ use constant {
 	F_DEAF		=> 2
 };
 
-our $bsnick = 'BotServ';
+our $bsnick_default = 'BotServ';
+our $bsnick = $bsnick_default;
 our $botchmode;
-if(!net::PREFIXAQ_DISABLE()) {
+if(!ircd::PREFIXAQ_DISABLE()) {
 	$botchmode = '+q';
 } else {
 	$botchmode = '+qo';
 }
 
-*notice = \&nickserv::notice;
 *agent = \&chanserv::agent;
 
+our $calc_safe = new Safe;
+
 our (
 	$get_all_bots, $get_botchans, $get_botstay_chans, $get_chan_bot, $get_bots_chans, $get_bot_info,
 
@@ -89,7 +97,7 @@ sub init() {
 sub dispatch($$$) {
         my ($src, $dst, $msg) = @_;
 	
-	if($dst =~ /^$bsnick$/i) {
+	if(lc $dst eq lc $bsnick or lc $dst eq lc $bsnick_default ) {
 		bs_dispatch($src, $dst, $msg);
 	}
 	elsif($dst =~ /^#/) {
@@ -117,9 +125,9 @@ sub bs_dispatch($$$) {
 	my @args = split(/\s+/, $msg);
 	my $cmd = shift @args;
 
-	my $user = { NICK => $src, AGENT => $bsnick };
+	my $user = { NICK => $src, AGENT => $dst };
 
-	return if operserv::flood_check($user);
+	return if flood_check($user);
 
 	if($cmd =~ /^assign$/i) {
 		if (@args == 2) {
@@ -198,7 +206,7 @@ sub bs_dispatch($$$) {
 		}
 	}
 	elsif($cmd =~ /^help$/i) {
-		help::sendhelp($bsnick, $src, 'botserv', @args);
+		sendhelp($user, 'botserv', @args);
 	}
 	elsif($cmd =~ /^d(ice)?$/i) {
 		notice($user, get_dice($args[0]));
@@ -215,7 +223,7 @@ sub bs_assign($$$) {
 
 	chanserv::chk_registered($user, $chan) or return;
 
-	unless (chanserv::can_do($chan, 'BotAssign', undef, $user)) {
+	unless (chanserv::can_do($chan, 'BotAssign', $user)) {
 		notice($user, $err_deny);
 		return;
 	}
@@ -305,10 +313,10 @@ sub bs_add($$$$$) {
 	}
 
 	$create_bot->execute($botnick, $botident, $bothost, $botgecos);
-	net::sqline($botnick, $services::qlreason);
+	ircd::sqline($botnick, $services::qlreason);
 	agent_connect($botnick, $botident, $bothost, '+pqBSrz', $botgecos);
-	agent_join($botnick, $config::conf{diag});
-	net::setmode($main::rsnick, $config::conf{diag}, '+h', $botnick);
+	agent_join($botnick, main_conf_diag);
+	ircd::setmode($main::rsnick, main_conf_diag, '+h', $botnick);
 
 	notice($user, "Bot $botnick connected.");
 }
@@ -329,7 +337,7 @@ sub bs_del($$) {
 	my $src = get_user_nick($user);
 	$delete_bot->execute($botnick);
 	agent_quit($botnick, "Deleted by \002$src\002.");
-	net::unsqline($botnick);
+	ircd::unsqline($botnick);
 	
 	$delete_bot_allchans->execute($botnick);
 	notice($user, "Bot \002$botnick\002 disconnected.");
@@ -445,10 +453,10 @@ sub bs_change($$$$$$) {
 	# then finally delete the old bot
 	# This is to prevent races.
 	$create_bot->execute($botnick, $botident, $bothost, $botgecos);
-	net::sqline($botnick, $services::qlreason);
+	ircd::sqline($botnick, $services::qlreason);
 	agent_connect($botnick, $botident, $bothost, '+pqBSrz', $botgecos);
-	agent_join($botnick, $config::conf{diag});
-	net::setmode($main::rsnick, $config::conf{diag}, '+h', $botnick);
+	agent_join($botnick, main_conf_diag);
+	ircd::setmode($main::rsnick, main_conf_diag, '+h', $botnick);
 
 	notice($user, "Bot $botnick connected.");
 
@@ -465,7 +473,7 @@ sub bs_change($$$$$$) {
 	my $src = get_user_nick($user);
 	$delete_bot->execute($oldnick);
 	agent_quit($oldnick, "Deleted by \002$src\002.");
-	net::unsqline($oldnick);
+	ircd::unsqline($oldnick);
 	notice($user, "Bot \002$oldnick\002 disconnected.");
 }
 
@@ -482,84 +490,103 @@ sub chan_dispatch($$$) {
 	my $user = { NICK => $src, AGENT => agent($chan) };
 
 	my %cmdhash = (
-		'voice' 	=>	'give_ops',
-		'devoice' 	=>	'give_ops',
-		'hop'	 	=>	'give_ops',
-		'halfop' 	=>	'give_ops',
-		'dehop' 	=>	'give_ops',
-		'dehalfop' 	=>	'give_ops',
-		'op'	 	=>	'give_ops',
-		'deop' 		=>	'give_ops',
-		'protect' 	=>	'give_ops',
-		'admin'		=>	'give_ops',
-		'deprotect' 	=>	'give_ops',
-		'deadmin'	=>	'give_ops',
+		'voice'		=>	\&give_ops,
+		'devoice'	=>	\&give_ops,
+		'hop'		=>	\&give_ops,
+		'halfop'	=>	\&give_ops,
+		'dehop'		=>	\&give_ops,
+		'dehalfop'	=>	\&give_ops,
+		'op'		=>	\&give_ops,
+		'deop'		=>	\&give_ops,
+		'protect'	=>	\&give_ops,
+		'admin'		=>	\&give_ops,
+		'deprotect'	=>	\&give_ops,
+		'deadmin'	=>	\&give_ops,
 
-		'up'		=>	'up',
+		'up'		=>	\&up,
 
-		'down'		=>	'down',
-		'molest'	=>	'down',
+		'down'		=>	\&down,
+		'molest'	=>	\&down,
 
-		'invite'	=>	'invite',
+		'invite'	=>	\&invite,
 
-		'kick'		=>	'kick',
-		'k'		=>	'kick',
+		'kick'		=>	\&kick,
+		'k'		=>	\&kick,
 
-		'kb'		=>	'kickban',
-		'kickb'		=>	'kickban',
-		'kban'		=>	'kickban',
-		'kickban'	=>	'kickban',
-		'bk'		=>	'kickban',
-		'bkick'		=>	'kickban',
-		'bank'		=>	'kickban',
-		'bankick'	=>	'kickban',
+		'kb'		=>	\&kickban,
+		'kickb'		=>	\&kickban,
+		'kban'		=>	\&kickban,
+		'kickban'	=>	\&kickban,
+		'bk'		=>	\&kickban,
+		'bkick'		=>	\&kickban,
+		'bank'		=>	\&kickban,
+		'bankick'	=>	\&kickban,
 
-		'kickmask'	=>	'kickmask',
-		'km'		=>	'kickmask',
-		'kmask'		=>	'kickmask',
+		'kickmask'	=>	\&kickmask,
+		'km'		=>	\&kickmask,
+		'kmask'		=>	\&kickmask,
 
-		'kickbanmask'	=>	'kickbanmask',
-		'kickbmask'	=>	'kickbanmask',
-		'kickbm'	=>	'kickbanmask',
-		'kbm'		=>	'kickbanmask',
-		'kbanm'		=>	'kickbanmask',
-		'kbanmask'	=>	'kickbanmask',
-		'kbmask'	=>	'kickbanmask',
+		'kickbanmask'	=>	\&kickbanmask,
+		'kickbmask'	=>	\&kickbanmask,
+		'kickbm'	=>	\&kickbanmask,
+		'kbm'		=>	\&kickbanmask,
+		'kbanm'		=>	\&kickbanmask,
+		'kbanmask'	=>	\&kickbanmask,
+		'kbmask'	=>	\&kickbanmask,
 
-		'calc'		=>	'calc',
+		'calc'		=>	\&calc,
 
-		'seen'		=>	'seen',
+		'seen'		=>	\&seen,
 
 		#We really need something that is mostly obvious
 		# and won't be used by any other bots.
 		#TriviaBot I added !trivhelp
 		# I guess anope uses !commands
-		'help'		=>	'help',
-		'commands'	=>	'help',
-		'botcmds'	=>	'help',
+		'help'		=>	\&help,
+		'commands'	=>	\&help,
+		'botcmds'	=>	\&help,
 
-		'users'		=>	'alist',
-		'alist'		=>	'alist',
+		'abbrevs'	=>	\&help,
+		'abbreviations'	=>	\&help,
+		'abbrev'	=>	\&help,
 
-		'unban'		=>	'unban',
+		'users'		=>	\&alist,
+		'alist'		=>	\&alist,
 
-		'ban'		=>	'ban',
-		'b'		=>	'ban',
+		'unban'		=>	\&unban,
 
-		'd'		=>	'dice',
-		'dice'		=>	'dice',
+		'banlist'	=>	\&banlist,
+		'blist'		=>	\&banlist,
+
+		'ban'		=>	\&ban,
+		'b'		=>	\&ban,
+		'qban'		=>	\&ban,
+		'nban'		=>	\&ban,
+
+		'd'		=>	\&dice,
+		'dice'		=>	\&dice,
+
+		'mode'		=>	\&mode,
+		'm'		=>	\&mode,
+
+		'resync'	=>	\&resync,
+
+		'topic'		=>	\&topic,
+		't'		=>	\&topic,
+
+		'why'		=>	\&why,
 	);
 
 	sub give_ops {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		chanserv::cs_setmodes($user, $cmd, $chan, @args);
 	}
 	sub up {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		chanserv::cs_updown($user, $cmd, $chan->{CHAN}, @args);
 	}
 	sub down {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		if(lc $cmd eq 'molest') {
 			chanserv::unset_modes($user, $chan);
 		} else {
@@ -568,34 +595,34 @@ sub chan_dispatch($$$) {
 	}
 
 	sub invite {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		chanserv::cs_invite($user, $chan, @args) unless @args == 0;
 	}
 
 	sub kick {
-		my ($user, $cmd, $chan, @args) = @_;
-		my $target = shift @args;
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		my $target = shift @args or return;
 		chanserv::cs_kick($user, $chan, $target, 0, join(' ', @args));
 	}
 	sub kickban {
-		my ($user, $cmd, $chan, @args) = @_;
-		my $target = shift @args;
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		my $target = shift @args or return;
 		chanserv::cs_kick($user, $chan, $target, 1, join(' ', @args));
 	}
 
 	sub kickmask {
-		my ($user, $cmd, $chan, @args) = @_;
-		my $target = shift @args;
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		my $target = shift @args or return;
 		chanserv::cs_kickmask($user, $chan, $target, 0, join(' ', @args));
 	}
 	sub kickbanmask {
-		my ($user, $cmd, $chan, @args) = @_;
-		my $target = shift @args;
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		my $target = shift @args or return;
 		chanserv::cs_kickmask($user, $chan, $target, 1, join(' ', @args));
 	}
 
 	sub calc {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		my $msg = join(' ', @args);
 		for ($msg) {
 			s/,/./g;
@@ -604,12 +631,15 @@ sub chan_dispatch($$$) {
 			s/\^/**/g;
 			s/(?<!0)x//g;
 		}
-		my $answer = eval("($msg) || 0");
+
+		my $answer = $calc_safe->reval("($msg) || 0");
+		$answer = 'ERROR' unless defined $answer;
+
 		notice($user, ($@ ? "$msg = ERROR (${\ (split / at/, $@, 2)[0]})" : "$msg = $answer"));
 	}
 
 	sub seen {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		
 		if(@args >= 1) {
 			nickserv::ns_seen($user, @args);
@@ -619,17 +649,21 @@ sub chan_dispatch($$$) {
 	}
 
 	sub help {
-		my ($user, $cmd, $chan, @args) = @_;
-		help::sendhelp(agent($chan), get_user_nick($user), 'chanbot');
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		if($cmd =~ /^abbrev(iation)?s?$/) {
+			sendhelp($user, 'chanbot', 'abbreviations');
+		} else {
+			sendhelp($user, 'chanbot');
+		}
 	}
 
 	sub alist {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		chanserv::cs_alist($user, $chan);
 	}
 
 	sub unban {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
 		if(@args == 0) {
 			chanserv::cs_unban($user, $chan, get_user_nick($user));
 		}
@@ -639,27 +673,61 @@ sub chan_dispatch($$$) {
 	}
 
 	sub ban {
-		my ($user, $cmd, $chan, @args) = @_;
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		$cmd =~ /^(q|n)?ban$/; my $type = $1;
 		if(@args >= 1) {
-			chanserv::cs_ban($user, $chan, @args);
+			chanserv::cs_ban($user, $chan, $type, @args);
 		}
 	}
 
+	sub banlist {
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		chanserv::cs_banlist($user, $chan);
+	}
+
 	sub dice {
 	# FIXME: If dice is disabled, don't count towards flooding.
-		my ($user, $cmd, $chan, @args) = @_;
-		
-		if(chanserv::can_do($chan, 'DICE', undef, $user)) {
-			net::privmsg(agent($chan), $chan->{CHAN},
+		my ($user, $chan, $cmd, undef, @args) = @_;
+
+		if(chanserv::can_do($chan, 'DICE', $user)) {
+			ircd::privmsg(agent($chan), $chan->{CHAN},
 				get_dice($args[0]));
 		}
 	}
 
+	sub mode {
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		if(@args >= 1) {
+			chanserv::cs_mode($user, $chan, shift @args, @args);
+		}
+	}
 
+	sub resync {
+		my ($user, $chan, $cmd) = @_;
+		chanserv::cs_resync($user, $chan->{CHAN});
+	}
+
+	sub topic {
+		my ($user, $chan, $cmd, $msg) = @_;
+		if (@args >= 1) {
+			$msg =~ s/^!$cmd //;
+			chanserv::cs_topic($user, $chan, $msg);
+		}
+	}
+
+	sub why {
+		my ($user, $chan, $cmd, undef, @args) = @_;
+		
+		if(@args >= 1) {
+			chanserv::cs_why($user, $chan, @args);
+		} else {
+			notice($user, 'Syntax: WHY <nick> [nick ...]');
+		}
+	}
 	if(defined($cmdhash{$cmd})) {
-		return if operserv::flood_check($user);
+		return if flood_check($user);
 
-		&{$cmdhash{$cmd}}($user, $cmd, $chan, @args);
+		&{$cmdhash{$cmd}}($user, $chan, $cmd, $msg, @args);
 	}
 }
 
@@ -667,12 +735,13 @@ sub bot_say($$$) {
 	my ($user, $chan, $botmsg) = @_;
 	my $cn = $chan->{CHAN};
 	
-	if(chanserv::can_do($chan, 'BotSay', undef, $user)) {
-		net::notice(agent($chan), '%'.$cn, get_user_nick($user).' used BotSay')
+	if(chanserv::can_do($chan, 'BotSay', $user)) {
+		ircd::notice(agent($chan), '%'.$cn, get_user_nick($user).' used BotSay')
 			if cr_chk_flag($chan, CRF_VERBOSE());
-		net::privmsg(agent($chan), $cn, $botmsg);
+		ircd::privmsg(agent($chan), $cn, $botmsg);
 	} else {
-		notice($user, $err_deny);
+		# can_do will give the $err_deny for us.
+		#notice($user, $err_deny);
 	}
 }
 
@@ -686,7 +755,7 @@ sub bot_dispatch($$$) {
     my $user = { NICK => $src, AGENT => $bot };
     my $chan = { CHAN => $cn };
 
-    return if operserv::flood_check($user);
+    return if flood_check($user);
     
     if ($cmd =~ /^join$/i) {
 	    if (adminserv::can_do($user, 'BOT')) {
@@ -710,22 +779,38 @@ sub bot_dispatch($$$) {
     }
     elsif ($cmd =~ /^help$/i) {
     	#my @help; @help = ($cn) if $cn; push @help, split(/\s+/, $botmsg);
-    	help::sendhelp($bot, $src, 'botpriv');
+    	sendhelp($user, 'botpriv');
     }
 }
 
 sub get_dice($) {
 	my ($count, $sides) = map int($_), ($_[0] ? split('d', $_[0]) : (1, 6));
 	
-	if ($sides < 1 or $sides > 1000 or $count < 1 or $count > 100) {
+	if ($sides < 1 or $sides > 1000 or $count < 0 or $count > 100) {
 		return "Sorry, you can't have more than 100 dice, or 1000 sides, or less than 1 of either.";
 	}
+	$count = 1 if $count == 0;
 	
 	my $sum = 0;
-	for(my $i = 1; $i <= $count; $i++) {
-		$sum += int(rand($sides)+1);
+
+	if($count == 1 or $count > 25) {
+		for(my $i = 1; $i <= $count; $i++) {
+			$sum += int(rand($sides)+1);
+		}
+
+		return "${count}d$sides: $sum";
+	}
+	else {
+		my @dice;
+
+		for(my $i = 1; $i <= $count; $i++) {
+			my $n = int(rand($sides)+1);
+			$sum += $n;
+			push @dice, $n;
+		}
+		
+		return "${count}d$sides: $sum  [" . join(' ', sort {$a <=> $b} @dice) . "]";
 	}
-	return "${count}d$sides: $sum";
 }
 
 ### IRC EVENTS ###
@@ -738,9 +823,9 @@ sub register() {
 	$get_all_bots->execute();
 	while(my ($nick, $ident, $vhost, $gecos, $flags) = $get_all_bots->fetchrow_array) {
 		agent_connect($nick, $ident, $vhost, '+pqBSrz'.(($flags & F_DEAF())?'d':''), $gecos);
-		net::sqline($nick, $services::qlreason);
-		agent_join($nick, $config::conf{diag});
-		net::setmode($main::rsnick, $config::conf{diag}, '+h', $nick);
+		ircd::sqline($nick, $services::qlreason);
+		agent_join($nick, main_conf_diag);
+		ircd::setmode($main::rsnick, main_conf_diag, '+h', $nick);
 	}
 }
 
@@ -754,7 +839,7 @@ sub eos() {
 		elsif(cr_chk_flag($chan, CRF_BOTSTAY(), 1)) {
 			bot_join($chan, $nick);
 			my $modelock = chanserv::get_modelock($chan);
-			net::setmode($config::conf{local}, $cn, $modelock) if $modelock;
+			ircd::setmode(main_conf_local, $cn, $modelock) if $modelock;
 		}
 	}
 }
@@ -782,7 +867,7 @@ sub bot_join($;$) {
 	
 	unless(is_agent_in_chan($nick, $cn)) {
 		agent_join($nick, $cn);
-		net::setmode($nick, $cn, $botchmode, $nick.(net::PREFIXAQ_DISABLE() ? ' '.$nick : '') );
+		ircd::setmode($nick, $cn, $botchmode, $nick.(ircd::PREFIXAQ_DISABLE() ? ' '.$nick : '') );
 	}
 }
 
