@@ -22,33 +22,38 @@ use SrSv::Errors;
 
 use SrSv::HostMask qw(parse_mask);
 
-use SrSv::User qw(get_user_nick get_user_id);
+use SrSv::User qw(get_user_nick get_user_id :flood);
+use SrSv::User::Notice;
+use SrSv::Help qw( sendhelp );
 
 use SrSv::NickReg::Flags qw(NRF_NOHIGHLIGHT nr_chk_flag_user);
 use SrSv::NickReg::User qw(is_identified);
 
 use SrSv::MySQL '$dbh';
 use SrSv::MySQL::Glob;
+require SrSv::DB::StubGen;
 
-*notice = \&nickserv::notice;
 
-our $hsnick = 'HostServ';
-
-our (
-	$set_vhost, $get_vhost, $del_vhost,
-	$vhost_chgroot,
-
-	$get_matching_vhosts
-);
+our $hsnick_default = 'HostServ';
+our $hsnick = $hsnick_default;
 
 sub init() {
-	$set_vhost = $dbh->prepare("REPLACE INTO vhost SELECT id, ?, ?, ?, UNIX_TIMESTAMP() FROM nickreg WHERE nick=?");
-	$get_vhost = $dbh->prepare("SELECT vhost.ident, vhost.vhost FROM vhost, nickalias WHERE nickalias.nrid=vhost.nrid AND nickalias.alias=?");
-	$del_vhost = $dbh->prepare("DELETE FROM vhost USING vhost, nickreg WHERE nickreg.nick=? AND vhost.nrid=nickreg.id");
+import SrSv::DB::StubGen (
+	dbh => $dbh,
+	generator => 'services_mysql_stubgen',
+);
 
-	$get_matching_vhosts = $dbh->prepare("SELECT nickreg.nick, vhost.ident, vhost.vhost, vhost.adder, vhost.time FROM
-		vhost, nickreg WHERE vhost.nrid=nickreg.id AND nickreg.nick LIKE ? AND vhost.ident LIKE ? AND vhost.vhost LIKE ? 
-		ORDER BY nickreg.nick");
+services_mysql_stubgen(
+	[set_vhost => 'INSERT', "REPLACE INTO vhost SELECT id, ?, ?, ?, UNIX_TIMESTAMP() FROM nickreg WHERE nick=?"],
+	[get_vhost => 'ROW',  "SELECT vhost.ident, vhost.vhost
+		FROM vhost, nickalias
+		WHERE nickalias.nrid=vhost.nrid AND nickalias.alias=?"],
+	[del_vhost => 'NULL', "DELETE FROM vhost USING vhost, nickreg WHERE nickreg.nick=? AND vhost.nrid=nickreg.id"],
+	[get_matching_vhosts => 'ARRAY', "SELECT nickreg.nick, vhost.ident, vhost.vhost, vhost.adder, vhost.time
+		FROM vhost JOIN nickreg ON (vhost.nrid=nickreg.id)
+		WHERE nickreg.nick LIKE ? AND vhost.ident LIKE ? AND vhost.vhost LIKE ?
+		ORDER BY nickreg.nick"],
+);
 }
 
 sub dispatch($$$) {
@@ -57,9 +62,9 @@ sub dispatch($$$) {
 	my @args = split(/\s+/, $msg);
 	my $cmd = shift @args;
 
-	my $user = { NICK => $src, AGENT => $hsnick };
+	my $user = { NICK => $src, AGENT => $dst };
 
-	return if operserv::flood_check($user);
+	return if flood_check($user);
 
 	if(lc $cmd eq 'on') {
 		hs_on($user, $src, 0);
@@ -92,7 +97,7 @@ sub dispatch($$$) {
 		}
 	}	
         elsif($cmd =~ /^help$/i) {
-                help::sendhelp($hsnick, $src, lc $hsnick, @args)
+		sendhelp($user, 'hostserv', @args)
         }
 	else { notice($user, "Unknown command."); }
 }
@@ -111,16 +116,15 @@ sub hs_on($$;$) {
 		return;
 	}
 	
-	$get_vhost->execute($nick);
-	my ($vident, $vhost) = $get_vhost->fetchrow_array;
+	my ($vident, $vhost) = get_vhost($nick);
 	unless ($vhost) {
 		notice($user, "You don't have a vHost.") unless $identify;
 		return;
 	}
 	if ($vident) {
-		net::chgident($hsnick, $src, $vident);
+		ircd::chgident($hsnick, $src, $vident);
 	}
-	net::chghost($hsnick, $src, $vhost);
+	ircd::chghost($hsnick, $src, $vhost);
 
 	notice($user, "Your vHost has been changed to \002".($vident?"$vident\@":'')."$vhost\002");
 }
@@ -129,11 +133,10 @@ sub hs_off($) {
 	my ($user) = @_;
 	my $src = get_user_nick($user);
 	
-	net::setumode($hsnick, $src, '-xt');
-	# doesn't work :(
-	#net::setumode($hsnick, $src, '+x');
-	
-	notice($user, "vHost disabled.", "To enable normal host cloaking, /mode $src +x");
+	# This requires a hack that is only known to work in UnrealIRCd 3.2.6 and later.
+	ircd::reset_cloakhost($hsnick, $src);
+
+	notice($user, "vHost reset to cloakhost.");
 }
 
 sub hs_sethost($$$) {
@@ -154,7 +157,7 @@ sub hs_sethost($$$) {
 	    ($vident, $vhost) = split(/\@/, $vhost);
 	}
 	my $src = get_user_nick($user);
-	$set_vhost->execute($vident, $vhost, $src, $rootnick);
+	set_vhost($vident, $vhost, $src, $rootnick);
 	
 	notice($user, "vHost for \002$target ($rootnick)\002 set to \002".($vident?"$vident\@":'')."$vhost\002");
 }
@@ -172,7 +175,7 @@ sub hs_delhost($$) {
 		return;
 	}
 
-	$del_vhost->execute($rootnick);
+	del_vhost($rootnick);
 	
 	notice($user, "vHost for \002$target ($rootnick)\002 deleted.");
 }
@@ -181,7 +184,7 @@ sub hs_list($$) {
 	my ($user, $mask) = @_;
 
 	unless(adminserv::is_svsop($user, adminserv::S_HELP())) {
-		net::notice($user, $err_deny);
+		notice($user, $err_deny);
 		return;
 	}
 
@@ -192,8 +195,8 @@ sub hs_list($$) {
 	$mhost = '%' if($mhost eq '');
 
 	my @data;
-	$get_matching_vhosts->execute($mnick, $mident, $mhost);
-	while(my ($rnick, $vident, $vhost) = $get_matching_vhosts->fetchrow_array) {
+	foreach my $vhostEnt (get_matching_vhosts($mnick, $mident, $mhost)) {
+		my ($rnick, $vident, $vhost) = @$vhostEnt;
 		push @data, [$rnick, ($vident?"$vident\@":'').$vhost];
 	}
 
