@@ -16,7 +16,6 @@
 package memoserv;
 
 use strict;
-use DBI qw(:sql_types);
 #use constant {
 #	READ => 1,
 #	DEL => 2,
@@ -30,20 +29,25 @@ use SrSv::Time;
 use SrSv::Text::Format qw(columnar);
 use SrSv::Errors;
 
-use SrSv::User qw(get_user_nick get_user_id);
+use SrSv::User qw(get_user_nick get_user_id get_user_agent :flood);
+use SrSv::User::Notice;
+use SrSv::Help qw( sendhelp );
+
+use SrSv::Conf2Consts qw( main );
 
 use SrSv::NickReg::Flags;
 use SrSv::NickReg::User qw(is_identified get_nick_user_nicks);
 
-use SrSv::MySQL '$dbh';
+use SrSv::MySQL qw( $dbh :sql_types );
 
-*notice = \&nickserv::notice;
+use SrSv::Util qw( makeSeqList seqifyList );
 
 use constant (
 	MAX_MEMO_LEN => 400
 );
 
-our $msnick = 'MemoServ';
+our $msnick_default = 'MemoServ';
+our $msnick = $msnick_default;
 
 our (
 	$send_memo, $send_chan_memo, $get_chan_recipients,
@@ -57,8 +61,8 @@ our (
 	$delete_memo, $purge_memos, $delete_all_memos,
 	$memo_chgroot,
 
-	$add_ignore, $del_ignore, $list_ignore, $chk_ignore, $count_ignore, $wipe_ignore,
-	$drop_ignore, $ignore_chgroot1, $ignore_chgroot2, $adjust_ignore, $consolidate_ignore
+	$add_ignore, $get_ignore_num, $del_ignore_nick, $list_ignore, $chk_ignore,
+	$wipe_ignore, $purge_ignore,
 );
 
 sub init() {
@@ -73,7 +77,8 @@ sub init() {
 
 	$get_memo_list = $dbh->prepare("SELECT memo.src, memo.chan, memo.time, memo.flag, memo.msg FROM memo, nickreg WHERE nickreg.nick=? AND memo.dstid=nickreg.id ORDER BY memo.time ASC");
 
-	$get_memo = $dbh->prepare("SELECT memo.src, memo.chan, memo.time FROM memo, nickreg WHERE nickreg.nick=? AND memo.dstid=nickreg.id ORDER BY memo.time ASC LIMIT 1 OFFSET ?");
+	$get_memo = $dbh->prepare("SELECT memo.src, memo.chan, memo.time 
+		FROM memo JOIN nickreg ON (memo.dstid=nickreg.id) WHERE nickreg.nick=? ORDER BY memo.time ASC LIMIT 1 OFFSET ?");
 	$get_memo->bind_param(2, 0, SQL_INTEGER);
 	$get_memo_full = $dbh->prepare("SELECT memo.src, memo.chan, memo.time, memo.flag, memo.msg FROM memo, nickreg WHERE nickreg.nick=? AND memo.dstid=nickreg.id ORDER BY memo.time ASC LIMIT 1 OFFSET ?");
 	$get_memo_full->bind_param(2, 0, SQL_INTEGER);
@@ -86,15 +91,30 @@ sub init() {
 	$purge_memos = $dbh->prepare("DELETE FROM memo USING memo, nickreg WHERE nickreg.nick=? AND memo.dstid=nickreg.id AND memo.flag=1");
 	$delete_all_memos = $dbh->prepare("DELETE FROM memo USING memo, nickreg WHERE nickreg.nick=? AND memo.dstid=nickreg.id");
 
-	$add_ignore = $dbh->prepare("INSERT INTO ms_ignore SELECT nickreg.id, ignorenick.id, UNIX_TIMESTAMP(), ? FROM nickreg, nickreg AS ignorenick WHERE nickreg.nick=? AND ignorenick.nick=?");
-	$del_ignore = $dbh->prepare("DELETE FROM ms_ignore USING ms_ignore, nickreg, nickreg AS ignorenick WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id AND ms_ignore.id=? OR ignorenick.nick=? AND ms_ignore.ignoreid=ignorenick.id");
-	$list_ignore = $dbh->prepare("SELECT ignorenick.nick, ms_ignore.time, ms_ignore.id FROM ms_ignore, nickreg, nickreg AS ignorenick WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id AND ms_ignore.ignoreid=ignorenick.id ORDER BY ms_ignore.id");
-	$chk_ignore = $dbh->prepare("SELECT 1 FROM ms_ignore, nickreg, nickreg AS ignorenick WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id AND ignorenick.nick=? AND ms_ignore.ignoreid=ignorenick.id");
-	$count_ignore = $dbh->prepare("SELECT COUNT(*) FROM ms_ignore, nickreg WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id");
-	$wipe_ignore = $dbh->prepare("DELETE FROM ms_ignore USING ms_ignore, nickreg WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id");
+	$add_ignore = $dbh->prepare("INSERT INTO ms_ignore (ms_ignore.nrid, ms_ignore.ignoreid, time)
+		SELECT nickreg.id, ignorenick.id, UNIX_TIMESTAMP() FROM nickreg, nickreg AS ignorenick
+		WHERE nickreg.nick=? AND ignorenick.nick=?");
+	$del_ignore_nick = $dbh->prepare("DELETE FROM ms_ignore USING ms_ignore
+		JOIN nickreg ON (ms_ignore.nrid=nickreg.id)
+		JOIN nickreg AS ignorenick ON(ms_ignore.ignoreid=ignorenick.id)
+		WHERE nickreg.nick=? AND ignorenick.nick=?");
+	$get_ignore_num = $dbh->prepare("SELECT ignorenick.nick FROM ms_ignore
+		JOIN nickreg ON (ms_ignore.nrid=nickreg.id)
+		JOIN nickreg AS ignorenick ON(ms_ignore.ignoreid=ignorenick.id)
+		WHERE nickreg.nick=?
+		ORDER BY ms_ignore.time LIMIT 1 OFFSET ?");
+	$get_ignore_num->bind_param(2, 0, SQL_INTEGER);
 
-	$adjust_ignore = $dbh->prepare("UPDATE ms_ignore, nickreg SET ms_ignore.id=ms_ignore.id+? WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id");
-	$consolidate_ignore = $dbh->prepare("UPDATE ms_ignore, nickreg SET ms_ignore.id=ms_ignore.id-1 WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id and ms_ignore.id>?");
+	$list_ignore = $dbh->prepare("SELECT ignorenick.nick, ms_ignore.time
+		FROM ms_ignore, nickreg, nickreg AS ignorenick
+		WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id AND ms_ignore.ignoreid=ignorenick.id
+		ORDER BY ms_ignore.time");
+	$chk_ignore = $dbh->prepare("SELECT 1
+		FROM ms_ignore, nickreg, nickreg AS ignorenick
+		WHERE nickreg.nick=? AND ms_ignore.nrid=nickreg.id AND ignorenick.nick=? AND ms_ignore.ignoreid=ignorenick.id");
+
+	$wipe_ignore = $dbh->prepare("DELETE FROM ms_ignore USING ms_ignore JOIN nickreg ON(ms_ignore.nrid=nickreg.id) WHERE nickreg.nick=?");
+	$purge_ignore = $dbh->prepare("DELETE FROM ms_ignore USING ms_ignore JOIN nickreg ON(ms_ignore.ignoreid=nickreg.id) WHERE nickreg.nick=?");
 }
 
 ### MEMOSERV COMMANDS ###
@@ -105,9 +125,12 @@ sub dispatch($$$) {
 	my @args = split(/\s+/, $msg);
 	my $cmd = shift @args;
 
-	my $user = { NICK => $src, AGENT => $msnick };
+	my $user = { NICK => $src, AGENT => $dst };
 
-	return if operserv::flood_check($user);
+	return if flood_check($user);
+	if($SrSv::IRCd::State::queue_depth > main_conf_highqueue && !adminserv::is_svsop($user)) {
+		notice($user, get_user_agent($user)." is too busy right now. Please try your command again later.");
+	}
 
 	if($cmd =~ /^send$/i) {
 		if(@args >= 2) {
@@ -137,10 +160,10 @@ sub dispatch($$$) {
 		ms_list($user);
 	}
 	elsif($cmd =~ /^del(ete)?$/i) {
-		if(@args == 1 and (lc($args[0]) eq 'all' or $args[0] > 0)) {
+		if(@args >= 1 and (lc($args[0]) eq 'all' or $args[0] > 0)) {
 			ms_delete($user, $args[0]);
 		} else {
-			notice($user, 'Syntax: DELETE <num|ALL>');
+			notice($user, 'Syntax: DELETE <num|num1-num2|ALL>');
 		}
 	}
 	elsif($cmd =~ /^ign(ore)?$/i) {
@@ -169,7 +192,7 @@ sub dispatch($$$) {
 		}
 	}
 	elsif($cmd =~ /^help$/i) {
-		help::sendhelp($msnick, $src, lc $msnick, @args);
+		sendhelp($user, 'memoserv', @args);
 	}
 	else {
 		notice($user, "Unrecognized command.  For help, type: \002/ms help\002");
@@ -194,9 +217,9 @@ sub ms_send($$$$) {
 			return;
 		}
 		
-		chanserv::can_do($chan, 'MEMO', undef, $user) or return;
+		my $srcnick = chanserv::can_do($chan, 'MEMO', $user) or return;
 
-		send_chan_memo($user, $chan, $msg, $level);
+		send_chan_memo($srcnick, $chan, $msg, $level);
 	} else {
 		nickserv::chk_registered($user, $dst) or return;
 		
@@ -223,21 +246,37 @@ sub ms_read($$) {
 
 	my $root = auth($user) or return;
 
+	my @nums;
 	if(lc($num) eq 'last') {
 		$get_memo_count->execute($root);
 		($num) = $get_memo_count->fetchrow_array;
+		if (!$num) {
+			notice($user, "Memo \002$num\002 not found.");
+			return;
+		}
+		@nums = ($num);
+	} else {
+		@nums = makeSeqList($num);
 	}
-	
-	$get_memo_full->execute($root, $num-1);
-	unless(($from, $chan, $time, $flag, $msg) = $get_memo_full->fetchrow_array) {
-		notice($user, "That memo does not exist.");
-		return;
-	}
-	$set_flag->execute(1, $from, $root, $chan, $time);
 
-	notice($user, "Memo \002$num\002 from \002$from\002 ".
-		($chan ? "to \002$chan\002 " : "to \002$root\002 ").
-		"at ".gmtime2($time), ' ', '  '.$msg, ' --');
+	my $count = 0;
+	my @reply;
+	while (my $num = shift @nums) {
+		if (++$count > 5) {
+			push @reply, "You can only read 5 memos at a time.";
+			last;
+		}
+		$get_memo_full->execute($root, $num-1);
+		unless(($from, $chan, $time, $flag, $msg) = $get_memo_full->fetchrow_array) {
+			push @reply, "Memo \002$num\002 not found.";
+			next;
+		}
+		$set_flag->execute(1, $from, $root, $chan, $time);
+		push @reply, "Memo \002$num\002 from \002$from\002 ".
+			($chan ? "to \002$chan\002 " : "to \002$root\002 ").
+			"at ".gmtime2($time), ' ', '  '.$msg, ' --';
+	}
+	notice($user, @reply);
 }
 
 sub ms_list($) {
@@ -267,26 +306,42 @@ sub ms_list($) {
 		NOHIGHLIGHT => nr_chk_flag_user($user, NRF_NOHIGHLIGHT) }, @data));
 }
 
-sub ms_delete($$) {
-	my ($user, $num) = @_;
+sub ms_delete($@) {
+	my ($user, @args) = @_;
 	my $src = get_user_nick($user);
 
 	my $root = auth($user) or return;
 
-	if(lc($num) eq 'all') {
+	if(scalar(@args) == 1 and lc($args[0]) eq 'all') {
 		$delete_all_memos->execute($root);
 		notice($user, 'All of your memos have been deleted.');
-	} else {
+		return;
+	}
+	my (@deleted, @notDeleted);
+	foreach my $num (reverse makeSeqList(@args)) {
+		if(int($num) ne $num) { # can this happen, given makeSeqList?
+			notice($user, "\002$num\002 is not an integer number");
+			next;
+		}
 		my ($from, $chan, $time);
 		$get_memo->execute($root, $num-1);
-		unless(($from, $chan, $time) = $get_memo->fetchrow_array) {
-	                notice($user, "That memo does not exist.");
-	                return;
-	        }
-		$delete_memo->execute($from, $root, $chan, $time);
-		notice($user, "Memo $num has been deleted.");
+		if(my ($from, $chan, $time) = $get_memo->fetchrow_array) {
+			$delete_memo->execute($from, $root, $chan, $time);
+			push @deleted, $num;
+		} else {
+			push @notDeleted, $num;
+		}
 	}
-}		
+	if(scalar(@deleted)) {
+		my $plural = (scalar(@deleted) == 1);
+		my $msg = sprintf("Memo%s deleted: ".join(', ', seqifyList @deleted), ($plural ? '' : 's'));
+		notice($user, $msg);
+	}
+	if(scalar(@notDeleted)) {
+		my $msg = sprintf("Memos not found: ".join(', ', seqifyList @notDeleted));
+		notice($user, $msg);
+	}
+}
 
 sub ms_ignore_add($$) {
 	my ($user, $nick) = @_;
@@ -305,11 +360,7 @@ sub ms_ignore_add($$) {
 
 	my $srcroot = nickserv::get_root_nick($src);
 
-	$count_ignore->execute($srcroot);
-	my $count = $count_ignore->fetchrow_array;
-	$count_ignore->finish();
-
-	$add_ignore->execute(++$count, $srcroot, $nickroot);
+	$add_ignore->execute($srcroot, $nickroot);
 
 	notice($user, "\002$nick\002 (\002$nickroot\002) added to \002$src\002 (\002$srcroot\002) memo ignore list.");
 }
@@ -324,23 +375,18 @@ sub ms_ignore_del($$) {
 	}
 	my $srcroot = nickserv::get_root_nick($src);
 
+	my $ignorenick;
 	if (misc::isint($entry)) {
-		my $ret = $del_ignore->execute($srcroot, $entry, undef);
-		if($ret == 1) {
-			notice($user, "Delete succeeded for ($srcroot): #$entry");
-		}
-		else {
-			notice($user, "Delete failed for ($srcroot): #$entry. entry does not exist?");
-		}
+		$get_ignore_num->execute($srcroot, $entry - 1);
+		($ignorenick) = $get_ignore_num->fetchrow_array();
+		$get_ignore_num->finish();
+	}
+	my $ret = $del_ignore_nick->execute($srcroot, ($ignorenick ? $ignorenick : $entry));
+	if($ret == 1) {
+		notice($user, "Delete succeeded for ($srcroot): $entry");
 	}
 	else {
-		my $ret = $del_ignore->execute($srcroot, undef, $entry);
-		if($ret == 1) {
-			notice($user, "Delete succeeded for ($srcroot): $entry");
-		}
-		else {
-			notice($user, "Delete failed for ($srcroot): $entry. entry does not exist?");
-		}
+		notice($user, "Delete failed for ($srcroot): $entry. entry does not exist?");
 	}
 }
 
@@ -403,16 +449,17 @@ sub send_memo($$$) {
 }
 
 sub send_chan_memo($$$$) {
-	my ($user, $chan, $msg, $level) = @_;
+	my ($src, $chan, $msg, $level) = @_;
 	my $cn = $chan->{CHAN};
-	my $src = get_user_nick($user);
+	$src = (is_agent($src) ? $src : nickserv::get_root_nick($src));
 
 	$send_chan_memo->execute($src, $cn, time(), $msg, $cn, $level);
 	# "INSERT INTO memo SELECT ?, nick, ?, ?, 0, ? FROM chanacc WHERE chan=? AND level >= ?"
 	
 	$get_chan_recipients->execute($cn, $level);
 	while(my ($u) = $get_chan_recipients->fetchrow_array) {
-		net::notice($msnick, $u, "You have a new memo from \002$src\002 to \002$cn\002.  To read it, type: \002/ms read last\002");
+		notice({ NICK => $u, AGENT => $msnick }, 
+			"You have a new memo from \002$src\002 to \002$cn\002.  To read it, type: \002/ms read last\002");
 	}
 }
 
@@ -420,7 +467,7 @@ sub notice_all_nicks($$) {
 	my ($nick, $msg) = @_;
 
 	foreach my $u (get_nick_user_nicks $nick) {
-		net::notice($msnick, $u, $msg);
+		notice({ NICK => $u, AGENT => $msnick }, $msg);
 	}
 }
 
