@@ -20,27 +20,33 @@ use strict;
 use SrSv::Timer qw(add_timer);
 
 use SrSv::IRCd::State qw(get_server_state);
-use SrSv::Unreal::Validate qw(valid_server);
+use SrSv::IRCd::Validate qw( valid_server valid_nick );
 
 use SrSv::Time;
 use SrSv::Text::Format qw(columnar);
 use SrSv::Errors;
+use SrSv::Log;
 
-use SrSv::User qw(get_user_nick get_user_id is_online);
+use SrSv::Conf2Consts qw(main services);
+
+use SrSv::User qw(get_user_nick get_user_id get_user_agent is_online get_user_info get_user_ip :flood);
+use SrSv::User::Notice;
+use SrSv::Help qw( sendhelp );
 
 use SrSv::NickReg::Flags qw(NRF_NOHIGHLIGHT nr_chk_flag_user);
 
 use SrSv::MySQL '$dbh';
 
+use SrSv::IPv6;
+
 use constant {
 	MAX_LIM => 16777215
 };
 
-*get_user_agent = \&nickserv::get_user_agent;
-*notice = \&nickserv::notice;
 *kill_user = \&nickserv::kill_user;
 
-our $osnick = 'OperServ';
+our $osnick_default = 'OperServ';
+our $osnick = $osnick_default;
 
 my %newstypes = (
 	u => 'User',
@@ -63,9 +69,11 @@ our (
 	$del_clone_exceptname, $del_clone_exceptip,
 	$list_clone_exceptname, $list_clone_exceptserver, $list_clone_exceptip,
 
-	$flood_check, $flood_inc, $flood_expire,
+	$get_clones_fromhost, $get_clones_fromnick, $get_clones_fromid, $get_clones_fromipv4,
 
-	$get_session_list
+	$get_session_list,
+
+	$get_newusers, $get_newusers_noid
 );
 
 sub init() {
@@ -106,11 +114,27 @@ sub init() {
 	$list_clone_exceptserver = $dbh->prepare("SELECT host, adder, lim FROM sesexname WHERE serv=1 ORDER BY host ASC");
 	$list_clone_exceptip = $dbh->prepare("SELECT INET_NTOA(ip), mask, adder, lim FROM sesexip ORDER BY ip ASC");
 
-	$flood_check = $dbh->prepare("SELECT flood FROM user WHERE id=?");
-	$flood_inc = $dbh->prepare("UPDATE user SET flood = flood + ? WHERE id=?");
-	$flood_expire = $dbh->prepare("UPDATE user SET flood = flood >> 1"); # shift is faster than mul
+	$get_clones_fromhost = $dbh->prepare("SELECT user.nick, user.id, user.online
+		FROM user JOIN user AS clone ON (user.ip=clone.ip)
+		WHERE clone.host=? GROUP BY id");
+	$get_clones_fromnick = $dbh->prepare("SELECT user.nick, user.id, user.online
+		FROM user JOIN user AS clone ON (user.ip=clone.ip)
+		WHERE clone.nick=? GROUP BY id");
+	$get_clones_fromid = $dbh->prepare("SELECT user.nick, user.id, user.online
+		FROM user JOIN user AS clone ON (user.ip=clone.ip)
+		WHERE clone.id=? GROUP BY id");
+	$get_clones_fromipv4 = $dbh->prepare("SELECT user.nick, user.id, user.online
+		FROM user JOIN user AS clone ON (user.ip=clone.ip)
+		WHERE clone.ip=INET_ATON(?) GROUP BY id");
 
 	$get_session_list = $dbh->prepare("SELECT host, COUNT(*) AS c FROM user WHERE online=1 GROUP BY host HAVING c >= ?");
+
+	$get_newusers = $dbh->prepare("SELECT user.nick, user.id, user.online
+		FROM user
+		WHERE user.time > ?");
+	$get_newusers_noid = $dbh->prepare("SELECT user.nick, user.id, user.online
+		FROM user LEFT JOIN nickid ON (nickid.id=user.id)
+		WHERE nickid.id IS NULL AND user.time > ?");
 }
 
 sub dispatch($$$) {
@@ -119,14 +143,17 @@ sub dispatch($$$) {
 	my @args = split(/\s+/, $msg);
 	my $cmd = shift @args;
 
-	my $user = { NICK => $src, AGENT=> $osnick };
+	my $user = { NICK => $src, AGENT=> $dst };
 
-	services::ulog($osnick, log::INFO(), "cmd: [$msg]", $user);
+	services::ulog($osnick, LOG_INFO(), "cmd: [$msg]", $user);
 
 	return if flood_check($user);
 	unless(adminserv::is_svsop($user) or adminserv::is_ircop($user)) {
 		notice($user, $err_deny);
-		net::globops($osnick, "\002$src\002 failed access to $osnick $msg");
+		if($cmd =~ /^set/i) {
+			nickserv::kill_user($user, "OS SET doesn't exist here");
+		}
+		ircd::globops($osnick, "\002$src\002 failed access to $osnick $msg");
 		return;
 	}
 
@@ -140,12 +167,12 @@ sub dispatch($$$) {
 			if(@args >= 3 and $args[0] =~ /^\+/) {
 				@args = split(/\s+/, $msg, 5);
 				
-				os_qline_add($user, $args[2], $args[3], $args[4]);
+				os_qline_add($user, @args[2..4]);
 			}
 			elsif(@args >= 2) {
 				@args = split(/\s+/, $msg, 4);
 				
-				os_qline_add($user, 0, $args[2], $args[3]);
+				os_qline_add($user, 0, @args[2..3]);
 			}
 			else {
 				notice($user, 'Syntax: QLINE ADD [+expiry] <mask> <reason>');
@@ -177,9 +204,10 @@ sub dispatch($$$) {
 		}
 	}
 	elsif ($cmd =~ /^uinfo$/i)	{ os_uinfo($user, @args); }
+	elsif ($cmd =~ /^ninfo$/i)	{ os_ninfo($user, @args); }
 	elsif ($cmd =~ /^svsnick$/i)	{ os_svsnick($user, $args[0], $args[1]); }
 	elsif ($cmd =~ /^gnick$/i)	{ os_gnick($user, @args); }
-	elsif ($cmd =~ /^help$/i)	{ help::sendhelp($osnick, $src, 'operserv', @args) }
+	elsif ($cmd =~ /^help$/i)	{ sendhelp($user, 'operserv', @args) }
 	elsif ($cmd =~ /^(staff|listadm)$/i)	{ adminserv::as_staff($user) }
 	elsif ($cmd =~ /^logonnews$/i) {
 		my $cmd2 = shift @args;
@@ -341,6 +369,31 @@ sub dispatch($$$) {
 			notice($user, 'Syntax KILL <target> <reason here>');
 		}
 	}
+	elsif ($cmd =~ /^clones$/i) {
+		os_clones($user, @args);
+	}
+	elsif ($cmd =~ /^m(ass)?kill$/i) {
+		os_clones($user, 'KILL', @args);
+	}
+	elsif($cmd =~ /^(kline|gline)$/i) {
+	        if(@args >= 1) {
+	                os_gline($user, 0, @args);
+	        }
+	        else {
+			notice($user, 'Syntax GLINE <target> [+time] [reason here]');
+		}
+	}
+	elsif($cmd =~ /^(zline|gzline)$/i) {
+		if(@args >= 1) {
+			os_gline($user, 1, @args);
+		}
+		else {
+			notice($user, 'Syntax GZLINE <target> [+time] [reason here]');
+		}
+	}
+	elsif ($cmd =~ /^killnew$/i) {
+		os_killnew($user, @args);
+	}
 
 	else { notice($user, "Unknown command."); }
 }
@@ -359,7 +412,7 @@ sub os_fjoin($$@) {
 		notice($user, "You don't have the right access");
 		return $event::SUCCESS;
 	}
-	net::svsjoin($osnick, $target, @chans);
+	ircd::svsjoin($osnick, $target, @chans);
 }
 
 sub os_fpart($$@) {
@@ -383,7 +436,7 @@ sub os_fpart($$@) {
 	}
 	$reason = join(' ', @params) if @params;
 	
-	net::svspart($osnick, $target, $reason, @chans);
+	ircd::svspart($osnick, $target, $reason, @chans);
 }
 
 sub os_qline_add($$$$) {
@@ -402,7 +455,7 @@ sub os_qline_add($$$$) {
 	} else {
 		my $src = get_user_nick($user);
 		$add_qline->execute($src, $mask, $reason, time(), $expiry);
-		net::sqline($mask, $reason);
+		ircd::sqline($mask, $reason);
 		notice($user, "$mask is now Q:lined");
 	}
 }
@@ -415,7 +468,7 @@ sub os_qline_del($$) {
 	$check_qline->execute($mask);
 	if($check_qline->fetchrow_array) {
 		$del_qline->execute($mask);
-		net::unsqline($mask);
+		ircd::unsqline($mask);
 		notice($user, "$mask unqlined");
 	} else {
 		notice($user, "$mask is not qlined");
@@ -471,7 +524,7 @@ sub os_jupe($$$) {
 		return $event::SUCCESS;
 	}
 
-	net::jupe_server($server, "Juped by ".get_user_nick($user).": $reason");
+	ircd::jupe_server($server, "Juped by ".get_user_nick($user).": $reason");
 	notice($user, "$server is now juped.");
 	return $event::SUCCESS;
 }
@@ -496,7 +549,12 @@ sub os_uinfo($@) {
 	my ($user, @targets) = @_;
 
 	my @userlist;
+	my @reply;
 	foreach my $target (@targets) {
+		if(ref($target)) {
+			push @userlist, $target;
+			next;
+		}
 		if($target =~ /\,/) {
 			push @targets, split(',', $target);
 			next;
@@ -505,45 +563,69 @@ sub os_uinfo($@) {
 		my $tuser = { NICK => $target };
 		my $tuid = get_user_id($tuser);
 		unless ($tuid) {
-			notice($user, "\002$target\002: user not found");
+			push @reply, "\002$target\002: user not found";
 			next;
 		}
 		push @userlist, $tuser;
 	}
+	@targets = (); # drop this list now.
 
-	notice($user, get_uinfo($user, @userlist));
+	notice($user, @reply, get_uinfo($user, @userlist));
+	return $event::SUCCESS;
+}
+
+sub os_ninfo($@) {
+	my ($user, @targetsIn) = @_;
+
+	my (@targetsOut, @reply);
+	foreach my $target (@targetsIn) {
+		if(not nickserv::is_registered($target)) {
+			push @reply, "\002$target\002: is not registered.";
+		}
+		my @targets = SrSv::NickReg::User::get_nick_users_all($target);
+		if(scalar(@targets) == 0) {
+			push @reply, "\002$target\002: no user[s] online.";
+			next;
+		}
+		push @targetsOut, @targets;
+	}
+	@targetsIn = (); # drop this list now.
+	notice($user, @reply) if scalar(@reply);
+	if(scalar(@targetsOut)) {
+		return os_uinfo($user, @targetsOut);
+	}
 	return $event::SUCCESS;
 }
 
 sub os_svsnick($$$) {
-    my ($user, $curnick, $newnick) = @_;
-    my $tuser = { NICK => $curnick };
-    
-    if(!adminserv::is_svsop($user, adminserv::S_ROOT())) {
-	notice($user, $err_deny);
+	my ($user, $curnick, $newnick) = @_;
+	my $tuser = { NICK => $curnick };
+
+	if(!adminserv::is_svsop($user, adminserv::S_ROOT())) {
+		notice($user, $err_deny);
+		return $event::SUCCESS;
+	}
+	if ((!$curnick) or (!$newnick)) {
+		notice($user, "Syntax: SVSNICK <curnick> <newnick>");
+		return $event::SUCCESS;
+	}
+	if (!is_online($tuser)) {
+		notice($user, $curnick.' is not online.');
+		return $event::SUCCESS;
+	}
+	if (nickserv::is_online($newnick)) {
+		notice($user, $newnick.' already exists.');
+		return $event::SUCCESS;
+	}
+	nickserv::enforcer_quit($newnick);
+	ircd::svsnick($osnick, $curnick, $newnick);
+	notice($user, $curnick.' changed to '.$newnick);
 	return $event::SUCCESS;
-    }
-    if ((!$curnick) or (!$newnick)) {
-	notice($user, "Syntax: SVSNICK <curnick> <newnick>");
-	return $event::SUCCESS;
-    }
-    if (!is_online($tuser)) {
-	notice($user, $curnick.' is not online.');
-	return $event::SUCCESS;
-    }
-    if (nickserv::is_online($newnick)) {
-	notice($user, $newnick.' already exists.');
-	return $event::SUCCESS;
-    }
-    nickserv::enforcer_quit($newnick);
-    net::svsnick($osnick, $curnick, $newnick);
-    notice($user, $curnick.' changed to '.$newnick);
-    return $event::SUCCESS;
 }
 
 sub os_gnick($@) {
 	my ($user, @targets) = @_;
-    
+
 	if(!adminserv::can_do($user, 'QLINE')) {
 		notice($user, $err_deny);
 		return $event::SUCCESS;
@@ -727,7 +809,7 @@ sub os_except_hostname_del($$) {
 	my $hostmask = $hostname;
 	$hostmask =~ s/\*/\%/g;
 	my $ret = $del_clone_exceptname->execute($hostmask);
-	net::notice($osnick, $config::conf{diag}, "hostname: $hostname; hostmask: $hostmask");
+	ircd::notice($osnick, main_conf_diag, "hostname: $hostname; hostmask: $hostmask");
 	
 	if($ret == 1) {
 		notice($user, "\002$hostname\002 successfully deleted from the hostname exception list");
@@ -759,6 +841,7 @@ sub os_except_IP_del($$$) {
 
 	os_except_pre($user) or return 0;
 	
+	no warnings 'misc';
 	my ($IP, $mask) = split(/\//, $IP);
 	$mask = 32 unless $mask;
 	my $ret = $del_clone_exceptip->execute($IP);
@@ -831,90 +914,10 @@ sub os_rehash($;$) {
 	    return $event::SUCCESS;
 	}
 
-	net::rehash_all_servers($type);
+	ircd::rehash_all_servers($type);
 	return $event::SUCCESS;
 }
 
-sub os_loners($@) {
-	my ($user, @args) = @_;
-	my $cmd = shift @args;
-	my $noid;
-	if ($cmd =~ /(not?id|noidentify)/) {
-		$noid = 1;
-		$cmd = shift @args;
-	}
-	if (defined($args[0]) and $args[0] =~ /(not?id|noidentify)/) {
-		$noid = 1;
-		shift @args;
-	}
-
-	if(!$cmd or $cmd =~ /^list$/i) {
-		my @reply;
-		foreach my $tuser (chanserv::get_users_nochans($noid)) {
-			push @reply, get_user_nick($tuser);
-		}
-		notice($user, "Users in zero channels", @reply);
-	}
-	elsif($cmd =~ /^uinfo$/i) {
-		notice($user, get_uinfo($user, chanserv::get_users_nochans($noid)));
-	}
-	elsif($cmd =~ /^kill$/i) {
-		unless(adminserv::is_svsop($user, adminserv::S_OPER())) {
-			notice($user, $err_deny);
-			return;
-		}
-		foreach my $tuser (chanserv::get_users_nochans($noid)) {
-			$tuser->{AGENT} = $osnick;
-			nickserv::kill_user($tuser,
-				"Killed by \002".get_user_nick($user)."\002".
-				(@args ? ": ".join(' ', @args) : '')
-			);
-		}
-	}
-	elsif($cmd =~ /^kline$/i) {
-		unless(adminserv::is_svsop($user, adminserv::S_OPER())) {
-			notice($user, $err_deny);
-			return;
-		}
-		foreach my $tuser (chanserv::get_users_nochans($noid)) {
-			$tuser->{AGENT} = $osnick;
-			nickserv::kline_user($tuser, $services::conf{chankilltime},
-				"K:Lined by \002".get_user_nick($user)."\002".
-				(@args ? ": ".join(' ', @args) : '')
-			);
-		}
-	}
-	elsif($cmd =~ /^(msg|message|notice)$/i) {
-		notice($user, "Must have message to send") unless(@args);
-		foreach my $tuser (chanserv::get_users_nochans($noid)) {
-			$tuser->{AGENT} = $osnick;
-			notice($tuser, 
-				"Automated message from \002".get_user_nick($user),
-				join(' ', @args)
-			);
-		}
-	}
-	elsif($cmd =~ /^fjoin$/i) {
-		unless(adminserv::can_do($user, 'FJOIN')) {
-			notice($user, $err_deny);
-			return;
-		}
-
-		if ($args[0] !~ /^#/) {
-			notice($user, "\002".$args[0]."\002 is not a valid channel name");
-			return;
-		}
-
-		foreach my $tuser (chanserv::get_users_nochans($noid)) {
-			$tuser->{AGENT} = $osnick;
-			net::svsjoin($osnick, get_user_nick($tuser), $args[0]);
-		}
-	}
-	else {
-		notice($user, "Unknown LONERS command: $cmd",
-			'Syntax: OS LONERS [LIST|UINFO|MSG|FJOIN|KILL|KLINE] [NOID] [msg/reason]');
-	}
-}
 
 sub os_svskill($$$) {
 	my ($user, $targets, $reason) = @_;
@@ -932,7 +935,7 @@ sub os_svskill($$$) {
 			return $event::SUCCESS;
 		}
 
-		net::svskill($osnick, $target, $reason);
+		ircd::svskill($osnick, $target, $reason);
 	}
 
 	return $event::SUCCESS;
@@ -957,9 +960,219 @@ sub os_kill($$$) {
 		nickserv::kill_user($tuser, "Killed by ".get_user_nick($user).($reason ? ': '.$reason : ''));
 	}
 
+}
+
+sub os_gline($$$@) {
+	my ($user, $zline, $target, @args) = @_;
+
+	my $opernick;
+	return unless ($opernick = adminserv::is_svsop($user, adminserv::S_OPER));
+
+	my $expiry;
+	$expiry = parse_time(shift @args) if $args[0] =~ /^\+/;
+	my $reason = join(' ', @args);
+	$reason =~ s/^\:// if $reason;
+	my $remove;
+	if($target =~ /^-/) {
+		$remove = 1;
+		$target =~ s/^-//;
+	}
+
+	my ($ident, $host);
+	if($target =~ /\!/) {
+		notice($user, "Invalid G:line target \002$target\002");
+		return;
+	}
+	elsif($target =~ /^(\S+)\@(\S+)$/) {
+		($ident, $host) = ($1, $2);
+	} elsif($target =~ /\./) {
+		($ident, $host) = ('*', $target);
+	} elsif(valid_nick($target)) {
+		my $tuser = { NICK => $target };
+		unless(get_user_id($tuser)) {
+			notice($user, "Unknown user \002$target\002");
+			return;
+		}
+		unless($zline) {
+			(undef, $host) = nickserv::get_host($tuser);
+			$ident = '*';
+		} else {
+			$host = get_user_ip($tuser);
+			if ($host =~ /:/) {
+				$host = get_ipv6_64($host);
+			}
+		}
+	} else {
+		notice($user, "Invalid G:line target \002$target\002");
+		return;
+	}
+	unless($zline) {
+		if(!$remove) {
+			ircd::kline($opernick, $ident, $host, $expiry, $reason);
+		} else {
+			ircd::unkline($opernick, $ident, $host);
+		}
+
+	} else {
+		if($ident and $ident !~ /^\**$/) {
+			notice($user, "You cannot specify an ident in a Z:line");
+		}
+		elsif ($host =~ /^(?:\d{1,3}\.){3}(?:\d{1,3})/) {
+			# all is well, do nothing
+		}
+		elsif ($host =~ /^[0-9\/\*\?\.]+$/) {
+			# This may allow invalid CIDR, not sure.
+			# We're trusting our opers to not do stupid things.
+			# THIS MAY BE A SOURCE OF BUGS.
+
+			# all is well, do nothing
+		} elsif($host =~ /:/) {
+			#validating IPv6 addrs without using inet_pton and inet_ntop is a crapshoot
+			# for now, we do nothing.
+		} else {
+			notice($user, "Z:lines can only be placed on IPs or IP ranges");
+			return;
+		}
+		if(!$remove) {
+			ircd::zline($opernick, $host, $expiry, $reason);
+		} else {
+			ircd::unzline($opernick, $host);
+		}
+	}
+
 	return $event::SUCCESS;
 }
 
+sub os_loners($@) {
+	my ($user, @args) = @_;
+	my $cmd = shift @args;
+	my $noid;
+	if ($cmd =~ /(not?id|noidentify)/) {
+		$noid = 1;
+		$cmd = shift @args;
+	}
+	if (defined($args[0]) and $args[0] =~ /(not?id|noidentify)/) {
+		$noid = 1;
+		shift @args;
+	}
+
+	return __os_massmod($user, uc 'clones', $cmd, \&chanserv::get_users_nochans, $noid, @args);
+}
+sub os_clones($@) {
+	my ($user, @args) = @_;
+	my $cmd = shift @args;
+	my $target = shift @args;
+
+	return __os_massmod($user, 'Clones', $cmd, \&get_clones, $target, @args);
+}
+
+sub os_killnew($@) {
+	my ($user, @args) = @_;
+	my $cmd = shift @args;
+
+	my ($noid, $time);
+	if ($cmd =~ /(not?id|noidentify)/) {
+		$noid = 1;
+		$cmd = shift @args;
+	}
+	if (defined($args[0]) and $args[0] =~ /(not?id|noidentify)/) {
+		$noid = 1;
+		shift @args;
+	}
+	if(defined($args[0] and $args[0] =~ /^\+/)) {
+		$time = parse_time(shift @args);
+	}
+
+	return __os_massmod($user, 'killnew', $cmd, \&get_newusers, [$noid, $time], @args);
+}
+
+sub __os_massmod($$$$@) {
+	my ($user, $cmd0, $cmd1, $func, $arg, @args) = @_;
+	my $msg = join(' ', @args);
+
+	if($cmd1 =~ /^list$/i) {
+		my @data;
+		my $noun;
+		foreach my $tuser (&$func($arg)) {
+			push @data, [get_user_nick($tuser), (is_online($tuser) ? "\002Online\002" : "\002Offline\002")];
+		}
+		my $title;
+		if($cmd0 eq 'Clones') {
+			$title = "$cmd0 matching \002$arg\002";
+		} elsif($cmd0 eq 'Loners') {
+			$title = "$cmd0 ".($arg ? 'Not identified' : '');
+		} elsif($cmd0 eq 'Loners') {
+			$title = "New users ".($arg ? 'Not identified' : '');
+		}
+		notice($user, columnar {TITLE => $title,
+			NOHIGHLIGHT => nr_chk_flag_user($user, NRF_NOHIGHLIGHT)}, @data);
+	}
+	elsif($cmd1 =~ /^uinfo$/i) {
+		notice($user, get_uinfo($user, &$func($arg)));
+	}
+	elsif($cmd1 =~ /^kill$/i) {
+		unless(adminserv::can_do($user, 'KILL')) {
+			notice($user, $err_deny);
+			return;
+		}
+		foreach my $tuser (&$func($arg)) {
+			next unless is_online($tuser);
+			$tuser->{AGENT} = $osnick;
+			nickserv::kill_user($tuser,
+				"Killed by \002".get_user_nick($user)."\002".
+				($msg ? ": $msg" : '')
+			);
+		}
+	}
+	elsif($cmd1 =~ /^kline$/i) {
+		unless(adminserv::is_svsop($user, adminserv::S_OPER())) {
+			notice($user, $err_deny);
+			return;
+		}
+		foreach my $tuser (&$func($arg)) {
+			next unless is_online($tuser);
+			$tuser->{AGENT} = $osnick;
+			nickserv::kline_user($tuser, services_conf_chankilltime,
+				"K:Lined by \002".get_user_nick($user)."\002".
+				($msg ? ": $msg" : '')
+			);
+		}
+	}
+	elsif($cmd1 =~ /^(msg|message|notice)$/i) {
+		notice($user, "Must have message to send") unless(@args);
+		foreach my $tuser (&$func($arg)) {
+			next unless is_online($tuser);
+			$tuser->{AGENT} = $osnick;
+			notice($tuser,
+				"Automated message from \002".get_user_nick($user),
+				$msg
+			);
+		}
+	}
+	elsif($cmd1 =~ /^fjoin$/i) {
+		unless(adminserv::can_do($user, 'FJOIN')) {
+			notice($user, $err_deny);
+			return;
+		}
+
+		if ($args[0] !~ /^#/) {
+			notice($user, "\002".$args[0]."\002 is not a valid channel name");
+			return;
+		}
+
+		foreach my $tuser (&$func($arg)) {
+			next unless is_online($tuser);
+			my $cn = $msg; # not a message, most cases it is
+			$tuser->{AGENT} = $osnick;
+			ircd::svsjoin($osnick, get_user_nick($tuser), $cn);
+		}
+	}
+	else {
+		notice($user, "Unknown $cmd0 command: $cmd1",
+			"Syntax: OS $cmd0 [LIST|UINFO|MSG|FJOIN|KILL|KLINE] [msg/reason]");
+	}
+}
+
 ### MISCELLANEA ###
 
 sub do_news($$) {
@@ -980,7 +1193,7 @@ sub do_news($$) {
 		push @reply, "[$banner ".$months[$mon]." $mday $year] $msg";
 	}
 	$get_logonnews->finish();
-	net::notice($config::conf{local}, $nick, @reply) if scalar(@reply);
+	ircd::notice(main_conf_local, $nick, @reply) if scalar(@reply);
 }
 
 sub chk_auth($$) {
@@ -999,7 +1212,7 @@ sub expire(;$) {
 
 	$get_expired_qlines->execute();
 	while (my ($mask) = $get_expired_qlines->fetchrow_array() ) {
-		net::unsqline($mask);
+		ircd::unsqline($mask);
 		$del_qline->execute($mask);
 	}
 	$get_expired_qlines->finish();
@@ -1009,7 +1222,7 @@ sub expire(;$) {
 	$get_expired_akills->execute();
 	while (my ($mask) = $get_expired_akills->fetchrow_array() ) {
 		($ident, $host) = split('@', $mask);
-		net::unkline($osnick, $ident, $host);
+		ircd::unkline($osnick, $ident, $host);
 		$del_akill->execute($mask);
 	}
 	$get_expired_akills->finish();
@@ -1018,52 +1231,22 @@ sub expire(;$) {
 	$del_expired_logonnews->execute();
 }
 
-sub flood_expire(;$) {
-	add_timer('flood_expire', 10, __PACKAGE__, 'operserv::flood_expire');
-	$flood_expire->execute();
-	$flood_expire->finish();
-}
-
-sub flood_check($;$) {
-	my ($user, $amt) = @_;
-	$amt = 1 unless defined($amt);
-
-	$flood_inc->execute($amt, get_user_id($user)) unless adminserv::is_svsop($user, adminserv::S_HELP());
-	
-	my $flev = get_flood_level($user);
-	if($flev > 8) {
-		kill_user($user, "Flooding services.");
-		return 1;
-	}
-	elsif($flev > 6) {
-		notice($user, "You are flooding services.") if $amt == 1;
-		return 1;
-	}
-	else {
-		return 0;
-	}
-}
-
-sub get_flood_level($) {
-	my ($user) = @_;
-
-	$flood_check->execute(get_user_id($user));
-	my ($level) = $flood_check->fetchrow_array;
-	return $level;
-}
-
 sub get_uinfo($@) {
 	my ($user, @userlist) = @_;
 	my @reply;
 	foreach my $tuser (@userlist) {
-		my ($ident, $host, $vhost, $gecos, $server) = nickserv::get_user_info($tuser);
+		my ($ident, $host, $vhost, $gecos, $server, $signontime, $quittime) = get_user_info($tuser);
 		my $modes = nickserv::get_user_modes($tuser);
 		my $target = get_user_nick($tuser);
 
 		my ($curchans, $oldchans) = chanserv::get_user_chans_recent($tuser);
 	
 		my @data = (
-			["Status:", (nickserv::is_online($tuser) ? "Online" : "Offline")],
+			["Status:", (nickserv::is_online($tuser) ?
+				"Online (".gmtime2($signontime).')' :
+				"Offline (".gmtime2($quittime).')'
+				)
+			],
 			["ID Nicks:", join(', ', nickserv::get_id_nicks($tuser))],
 			["Channels:", join(', ', @$curchans)],
 			["Recently Parted:", join(', ', @$oldchans)],
@@ -1074,14 +1257,59 @@ sub get_uinfo($@) {
 			["Current Server:", $server],
 			["Modes:", $modes]
 		);
-		push @data, ["Country:", country::get_country_long(country::get_user_country($tuser))]
-			if module::is_loaded('country');
+		if(module::is_loaded('country')) {
+			push @data, ["Country:", country::get_user_country_long($tuser)];
+		} elsif(module::is_loaded('geoip')) {
+			push @data, ["Location:", geoip::stringify_location(geoip::get_user_location($tuser))];
+		}
+			
 		push @reply, columnar {TITLE => "User info for \002$target\002:",
 			NOHIGHLIGHT => nr_chk_flag_user($user, NRF_NOHIGHLIGHT)}, @data;
 	}
 	return @reply;
 }
 
+sub get_clones($) {
+	my ($targets) = @_;
+	my @users;
+	foreach my $target (split(',', $targets)) {
+		my $sth; # statement handle. You'll see what I'll do with it next!
+		if($target =~ /^(?:\d{1,3}\.){3}\d{1,3}$/) {
+			$sth = $get_clones_fromipv4;
+		} elsif($target =~ /\./) { # doesn't really work with localhost. oh well.
+			$sth = $get_clones_fromhost;
+		} else {
+			$sth = $get_clones_fromnick;
+		}
+
+		$sth->execute($target);
+		while(my ($nick, $id, $online) = $sth->fetchrow_array()) {
+			push @users, { NICK => $nick, ID => $id, ONLINE => $online };
+		}
+		$sth->finish();
+	}
+	return @users;
+}
+
+sub get_newusers($) {
+	my ($noid, $time) = @{$_[0]};
+	ircd::debug("get_newusers: $time");
+	my @users;
+	my $sth; # statement handle. You'll see what I'll do with it next!
+	if($noid) {
+		$sth = $get_newusers_noid;
+	} else {
+		$sth = $get_newusers;
+	}
+
+	$sth->execute(CORE::time()-$time);
+	while(my ($nick, $id, $online) = $sth->fetchrow_array()) {
+		push @users, { NICK => $nick, ID => $id, ONLINE => $online };
+	}
+	$sth->finish();
+	return @users;
+}
+
 ## IRC EVENTS ##
 
 1;
