@@ -28,21 +28,30 @@ use constant { # Need them up here, before anybody derefs them.
 	NETDUMP => 0,
 };
 
-use Cwd 'abs_path';
+use Cwd qw( abs_path getcwd );
 use File::Basename;
 
-our $prefix;
 BEGIN {
-	$prefix = dirname(abs_path($0));
-	chdir $prefix;
+	my %constants = (
+		CWD => getcwd(),
+		PREFIX => dirname(abs_path($0)),
+	);
+	require constant; import constant(\%constants);
 }
+# FIXME: remove the chdir call!
+chdir PREFIX;
+use lib PREFIX;
+
+die("Please don't run services as root!\n") if $< eq 0;
 
 use Getopt::Long;
 BEGIN {
 	my @debug_pkgs;
+	my $compile_only = 0;
 
 	GetOptions(
 		"debug:s" => \@debug_pkgs,
+		"compile" => \$compile_only,
 	);
 
 	if(@debug_pkgs) {
@@ -54,64 +63,81 @@ BEGIN {
 			$SrSv::Debug::debug_pkgs{$pkg} = 1;
 		}
 	}
+	import constant { COMPILE_ONLY => $compile_only };
 }
 
+use SrSv::Conf::main;
+
 use SrSv::OnIRC (1);
 
 use SrSv::Debug;
-use SrSv::Conf 'main';
+use SrSv::Log;
+use SrSv::Conf2Consts qw(main);
 
 use IO::Socket;
 use Carp;
 
-use libs::config;
+use SrSv::IRCd::Send; # <-- is package ircd
 use libs::misc;
 use libs::event;
-use libs::net;
-use libs::help;
 use libs::modes;
-use libs::log;
-use libs::email;
-use libs::fmt;
 use libs::module;
 
 use SrSv::Process::Init ();
-use SrSv::Process::Worker qw(spawn);
+use SrSv::Process::Worker qw(spawn write_pidfiles);
 use SrSv::Message qw(add_callback);
 use SrSv::Timer qw(begin_timer);
 
-use SrSv::Conf 'main';
-*conf = \%main_conf; #FIXME
+#*conf = \%main_conf; #FIXME
 
 STDOUT->autoflush(1);
+STDERR->autoflush(1);
 
 our $progname = 'SurrealServices';
-our $version = '0.4.1';
-our $extraversion = 'configured for UnrealIRCd 3.2.5 and up';
+our $version = '0.4.3.0';
+our $extraversion = 'configured for UnrealIRCd 3.2.8.1';
 
 #FIXME: Figure out where $rsnick belongs and update all references
 our $rsnick; *rsnick = \$core::rsnick;
 
 print "Starting $progname $version.\n";
 
-config::loadconfig();
+#config::loadconfig();
+
+{
+	use SrSv::DB::Schema;
+	my $schemaVer = check_schema();
+	my $newestSchema = find_newest_schema();
+	if($schemaVer != $newestSchema) {
+		print "Found schema version ($schemaVer). Expected ($newestSchema). Did you run db-setup.pl ?\n";
+		die;
+	}
+}
 
 module::load();
-net::serv_connect();
+exit() if COMPILE_ONLY;
+print "Connecting...";
+ircd::serv_connect();
+print " Connected.\n";
 
 unless(DEBUG) {
 	exit if fork;
 	close STDIN;
 	close STDOUT;
 	close STDERR;
+	open STDIN, '<', '/dev/null';
+	open STDOUT, '>', '/dev/null';
+	open STDERR, '>', '/dev/null';
 	setpgrp();
 }
 
-if($main_conf{procs}) {
-	for(1..$main_conf{procs}) { spawn(); }
+if(main_conf_procs) {
+	for(1..main_conf_procs) { spawn(); }
 }
+write_pidfiles();
 
 SrSv::Process::Init::do_init();
+
 module::begin();
 
 begin_timer();
