@@ -17,18 +17,23 @@
 #       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 use strict;
-use Cwd 'abs_path';
-use File::Basename;
 use File::stat;
 
-use SrSv::Time;
-# as libs::config is in dir ../libs/config.pm
-use lib "..";
-use libs::config;
-use libs::misc;
+BEGIN {
+	use Cwd qw( abs_path getcwd );
+	use File::Basename;
+	my %constants = (
+		CWD => getcwd(),
+		PREFIX => abs_path(dirname(abs_path($0)).'/..'),
+	);
+	require constant; import constant(\%constants);
+}
+chdir PREFIX;
+use lib PREFIX;
 
-my $prefix = dirname(dirname(abs_path($0)).'../');
-my $logdir = "$prefix/logs";
+use SrSv::Time;
+
+my $logdir = PREFIX.'/logs';
 my $chanlogdir = "$logdir/chanlogs";
 my $gzip = qx(which gzip); 
 my $bzip2 = qx(which bzip2);
@@ -37,13 +42,13 @@ chomp ($gzip, $bzip2);
 # This is based on an average observed from chanlogs.
 # Thankfully bzcat and bzgrep tend to be agnostic.
 my $bzip_threshold = 1000; 
+# if less than 100 bytes, don't bother to gzip.
+my $gzip_threshold = 100; 
 
-my %main_conf = config::readHash("$prefix/config/main.conf");
-
-opendir LOGDIR, $logdir.'/';
+opendir ((my $LOGDIR), $logdir.'/');
 
 my $i = 0; my @today = gmt_date();
-while (my $filename = readdir(LOGDIR)) {
+while (my $filename = readdir($LOGDIR)) {
 	next if $filename eq '..' or $filename =~ /\.(gz|bz2)$/ or !(-f "$logdir/$filename");
 	my $dir; my ($year, $month, $day);
 	if($filename =~ /^services.log-(\d{4})-(\d{2})-(\d{2})$/i) {
@@ -77,10 +82,24 @@ while (my $filename = readdir(LOGDIR)) {
 		mkdir "$dir/$year/$month";
 	}
 	rename "$logdir/$filename", "$dir/$year/$month/$filename";
-	system(((stat("$dir/$year/$month/$filename")->[7] > $bzip_threshold) ? $bzip2 : $gzip),
-		'-9vv', "$dir/$year/$month/$filename");
+	compressFile("$dir/$year/$month/$filename");
 	$i++;
 }
-closedir LOGDIR;
+
+sub compressFile($) {
+	my ($file) = @_;
+	my $fileStat = stat($file);
+	my $fileSize = $fileStat->[7];
+	my $compressor;
+	if($fileSize > $bzip_threshold) {
+		$compressor = $bzip2;
+	} elsif($fileSize < $gzip_threshold) {
+		return;
+	} else {
+		$compressor = $gzip;
+	}
+	system($compressor, '-9vv', $file);
+}
+closedir $LOGDIR;
 
 print "Processed $i logs\n";
