@@ -16,6 +16,10 @@
 use strict;
 use DBI;
 
+# Add tables to this list to be skipped
+# SrSv wants to skip the country table
+our %skipList = ( 'country' => 1, 'geoip' => 1, 'geolocation' => 1, 'georegion' => 1, 'opm' => 1 );
+
 use constant {
 	DROP_TABLE => 1,
 #	Default maximum packet size is 1MB
@@ -29,41 +33,46 @@ use constant {
 	# WARNING: Doing this with hundred megabyte tables
 	# will probably be slow, and possibly DoS your system
 	# with an Out of Memory condition.
-	LARGE_TABLES => 0,
+	LARGE_TABLES => 1,
 	# Most of the time, you don't want to preserve the contents
 	# of a MEMORY or HEAP table, since they're just temporary
 	# and would have been lost on a server restart anyway.
 	# Then again, maybe you want to keep them. If so, set this to 0.
 	# This does still save the schema.
-	SKIP_HEAP_DUMP => 1,
-	
+	SKIP_HEAP_DUMP => 0,
+
 	# This should only be used for debugging purposes
 	# as otherwise it throws junk into the output stream
 	VERBOSE => 0,
 };
 
-my $dbh;
+our $dbh;
+our $prefix;
+
+BEGIN {
+use Cwd qw( abs_path getcwd );
+use File::Basename;
+	$prefix = dirname(dirname(abs_path($0)).'../');
+	chdir $prefix;
+	import constant { PREFIX => $prefix, CWD => getcwd() };
+}
 
 # WARNING: for the generic case, this needs to be adapted
 # Either adapt the config file that you use,
 # or create a static hash table
 sub get_sql_conn {
 # These libs aren't needed for the generic case
-use Cwd 'abs_path';
-use File::Basename;
-use lib '..';
-use libs::config;
+use SrSv::Conf::sql;
+use SrSv::Conf2Consts qw( sql );
 
-	my $prefix = dirname(dirname(abs_path($0)).'../');
-	my %config = config::readHash("$prefix/config/sql.conf");
 	my %MySQL_config = (
-		'mysql-db' => $config{'mysql-db'},
-		'mysql-user' => $config{'mysql-user'},
-		'mysql-pass' => $config{'mysql-pass'}
+		'mysql-db' => sql_conf_mysql_db,
+		'mysql-user' => sql_conf_mysql_user,
+		'mysql-pass' => sql_conf_mysql_pass
 	);
 
 	$dbh = DBI->connect(
-		"DBI:mysql:".$MySQL_config{'mysql-db'}.":mysql_server_prepare=1",
+		"DBI:mysql:".$MySQL_config{'mysql-db'},
 		$MySQL_config{'mysql-user'},
 		$MySQL_config{'mysql-pass'},
 		{
@@ -111,10 +120,15 @@ sub get_data($$) {
 	foreach my $row (@$results) {
 		my $i = 0;
 		foreach my $element (@$row) {
-			if ($column_data->[++$i]->{TYPE_NAME} =~ /^BLOB$/i and
+			if ($column_data->[++$i]->{TYPE_NAME} =~ /^(TEXT|BLOB)$/i and
 				length($element))
 			{
 				$element = '0x' . unpack ('H*', $element);
+			}
+			elsif ($column_data->[$i]->{TYPE_NAME} =~ /int$/i and
+				length($element))
+			{
+				# do nothing
 			} else {
 				$element = $dbh->quote($element);
 			}
@@ -148,7 +162,12 @@ sub get_data_large($$) {
 			if ($column_data->[++$i]->{TYPE_NAME} =~ /^(TEXT|BLOB)$/i and
 			 	length($element))
 			{
-				$element = unpack ('H', $element);
+				$element = '0x' . unpack ('H*', $element);
+			}
+			elsif ($column_data->[$i]->{TYPE_NAME} =~ /int$/i and
+				length($element))
+			{
+				# do nothing
 			} else {
 				$element = $dbh->quote($element);
 			}
@@ -180,7 +199,11 @@ sub do_dump() {
 			my $schema;
 			($schema, $column_data) = get_schema($table);
 			print $schema."\n";
-			next TABLE if ((SKIP_HEAP_DUMP) and ($schema =~ /(ENGINE|TYPE)=(HEAP|MEMORY)/));
+			if ((SKIP_HEAP_DUMP) and 
+			    (($schema =~ /(ENGINE|TYPE)=(HEAP|MEMORY)/)) or ($skipList{lc $table})
+			) {
+			    next TABLE;
+			}
 		}
 
 		print "--\n-- Dumping data for table '$table'\n--\n".
